// FILE: DetailedHex.java

/* GEMpire (Generic Empire for the Masses), a java package to support internet oriented 
 * empire-like strategy games and associated utilities.
 * Copyright (C) 1998 Christopher L. Marshall
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * My current email is christopherlmarshall@yahoo.com.  My home address is:
 * 10006 Leamoore Lane,
 * Vienna VA, 22181
 * 
 * It is my hope that this modest effort might assist others in producing Java
 * programs to implement a varied host of empire-like games.
 */

package GEMpire;
import java.awt.Image;
import java.util.Vector;
import java.io.*;

public class DetailedHex{
   // Instance variables and clone() method.

      private Piece p;

      public Object clone(){
         return new DetailedHex(p);
      }

   public DetailedHex(Piece p){
      this.p= p;
   }

   public DetailedHex(BufferedReader in) throws IOException, NullPointerException{
      try {
         String line= in.readLine();
         if (line.equals("Piece")){
            p= new Piece(in);
         }
         else {
            Globals.reportError(
               "DetailedHex(BufferedReader): unknown DetailedHex type "+
               "(should be \"Piece\"): "+line
            );
         }
      }
      catch (IOException e){
         Globals.reportError("DetailedHex(BufferedReader): read failed "+e);
         throw e;
      }
      catch (NullPointerException e){
         Globals.reportError("DetailedHex(BufferedReader): read failed.  Most Likely the stream was malformed. "+e);
         throw e;
      }
   }

   public void writeCellUpdateTo(PrintWriter out, boolean isVisible, int player, boolean assumeVisible){
      if (!isVisible&&!p.isFixed()&&!assumeVisible){
         out.println("H");
         return;
      }

      out.println("D");
      out.println("Piece");
      p.writeCellUpdateTo(out,isVisible,player,assumeVisible);
   }

   public final Image getImage(){
      return p.getImage(); 
   }
   public int getOwner(){
      return p.getOwner(); 
   }
   public boolean empty(){return (p==null);}

   // Piece manipulation
   public Piece selectPiece(){
      if (p==null) Globals.exit("DetailedHex.getPiece(): p should never be null!");
      if (p.isFixed()) return p.getPiece();
      return p;
   }
   public Piece getPiece(){
      return p;
   }
   public Piece getPiece(int id){
      if (p.getId()==id){
         return p;
      }
      return p.getPiece(id);
   }
   public Vector getPieces(){
      if (p.containsPieces()) return p.getPieces();

      Vector tmp= new Vector();
      tmp.addElement(p);
      return tmp;
   }
   public boolean attachPiece(Piece p1){
      return p.attachPiece(p1);
   }
   public boolean detachPiece(Piece p1){
      if (p1==null) return false;
      if (p.equals(p1)){
         p= null;
         return true;
      }

      return p.detachPiece(p1);
   }
}

