// FILE: EmpireClient.java

/* GEMpire (Generic Empire for the Masses), a java package to support internet oriented 
 * empire-like strategy games and associated utilities.
 * Copyright (C) 1998 Christopher L. Marshall
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * My current email is christopherlmarshall@yahoo.com.  My home address is:
 * 10006 Leamoore Lane,
 * Vienna VA, 22181
 * 
 * It is my hope that this modest effort might assist others in producing Java
 * programs to implement a varied host of empire-like games.
 */

package GEMpire;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;

public class EmpireClient extends Thread{
   class MenuItemListener implements ActionListener{ 
      public void actionPerformed(ActionEvent event){ 
         MenuItem item= (MenuItem)event.getSource(); 
         if (item==exitItem){ 
            f.dispose(); 
            Globals.exit(""); 
         } 
         else if (item==newItem){ 
            System.out.println("newItem"); 
         } 
         else if (item==setPasswordItem){ 
            String fieldNames[]={"password"};
            String initialValues[]={""};
            int sizes[]={15};
            String x[]= UserPrompt.getStrings( 
               f, 
               "Connect to Server",
               fieldNames, 
               initialValues, 
               sizes
            );
            if (esc!=null)
               esc.sendSetPasswordMessage(x[0]);
            else 
               System.out.println("Can't set password: we aren't connected yet!");
         } 
         else if (item==connectItem){ 
            System.out.println("connect"); 
            String fieldNames[]={"host","port"};
            String initialValues[]={"",""};
            int sizes[]={15,15};
            String x[]= UserPrompt.getStrings( 
               f, 
               "Connect to Server",
               fieldNames, 
               initialValues, 
               sizes
            );
            System.out.println("host="+x[0]);
            System.out.println("port="+x[1]);

            // here's we go...
            try {
               connectToServer(x[0],x[1]);
            }
            catch (java.net.UnknownHostException e){
               System.err.println("couldn't find host");
            }
            catch (IOException e){
               System.err.println(
                  "some sort of IO error occurred while trying to connect"
               );
            }
         } 
         else if (item==saveItem){ 
            System.out.println("saveItem"); 
         } 
         else if (item==saveAsItem){ 
            System.out.println("saveAsItem"); 
         } 
         else if (item==openItem){ 
            System.out.println("openItem"); 
         } 
         else if (item==unloadItem){ 
            esc.unloadCommand(); 
         } 
         else if (item==buildPieceItem){ 
            esc.buildPieceCommand(); 
         } 
         else if (item==getMapItem){ 
            esc.requestMap(); 
         } 
         else if (item==endTurnItem){ 
            esc.sendEndTurnMessage(); 
         } 
         else if (item==reviewTurnItem){ 
            esc.sendReviewTurnMessage(); 
         } 
         else if (item==productionSelectModeItem){ 
            System.out.println("productionSelectModeItem"); 
            if (m!=null){ 
               m.setSelectMode(Map.PRODUCTION_SELECT_MODE); 
            } 
         } 
         else if (item==pieceSelectModeItem){ 
            System.out.println("pieceSelectModeItem"); 
            if (m!=null){ 
               m.setSelectMode(Map.PIECE_SELECT_MODE); 
            } 
         } 
         else if (item==saveGameItem){
            String fieldNames[]={"filename"};
            String initialValues[]={""};
            int sizes[]={15};
            String x[]= UserPrompt.getStrings( 
               f, 
               "Connect to Server",
               fieldNames, 
               initialValues, 
               sizes
            );
            if (esc!=null)
               esc.sendSaveGameMessage(x[0]);
            else 
               System.out.println("Can't save game: we aren't connected yet!");
         }
         else if (item==attackerAlwaysWinsItem){
            esc.sendAttackerAlwaysWinsMessage(); 
            System.out.println("attackerAlwaysWinsItem"); 
         } 
         else if (item==defenderAlwaysWinsItem){
            esc.sendDefenderAlwaysWinsMessage(); 
            System.out.println("defenderAlwaysWinsItem"); 
         } 
         else if (item==fairCombatItem){
            esc.sendFairCombatMessage(); 
            System.out.println("fairCombatItem");
         } 
      } 
   } 
 
   // instance variables
   EmpireServerConnection esc;
   Frame f = new Frame("Empire Client");
   Map m;

   BufferedReader in;
   PrintWriter out;

   private MenuBar mbar;
   MenuItem newItem,openItem,setPasswordItem,connectItem,saveAsItem,saveItem,exitItem;
   MenuItem unloadItem,buildPieceItem,getMapItem,endTurnItem,reviewTurnItem,
      productionSelectModeItem,pieceSelectModeItem;
   MenuItem saveGameItem,attackerAlwaysWinsItem,defenderAlwaysWinsItem,fairCombatItem; 

   Label statusBar= new Label("Hi there!");

   // contructors
   public EmpireClient(BufferedReader in, PrintWriter out){ 
      super(); 
      this.in= in; 
      this.out= out; 
      initMenuBar(); 
   } 
 
   public EmpireClient() throws IOException{
      super();
      initMenuBar();
   }
 
   public EmpireClient(InetAddress addr, int port) throws IOException{ 
      super(); 
      Socket s= new Socket(addr,port); 
      in= new BufferedReader(new InputStreamReader(s.getInputStream())); 
      out= new PrintWriter(new OutputStreamWriter(s.getOutputStream())); 
      initMenuBar(); 
   } 
 
   private void initMenuBar(){
      MenuItemListener mlist= new MenuItemListener();
      mbar= new MenuBar();

      Menu fileMenu= new Menu("File",true);
      fileMenu.add(newItem=    new MenuItem("New"));
      fileMenu.add(openItem=   new MenuItem("Open")); 
      fileMenu.add(setPasswordItem=   new MenuItem("set Password")); 
      fileMenu.add(connectItem=   new MenuItem("Connect")); 
      fileMenu.add(saveAsItem= new MenuItem("save As"));
      fileMenu.add(saveItem=   new MenuItem("Save"));
      fileMenu.addSeparator();
      fileMenu.add(exitItem=   new MenuItem("Exit"));
      newItem.addActionListener(mlist);
      openItem.addActionListener(mlist); 
      setPasswordItem.addActionListener(mlist); 
      connectItem.addActionListener(mlist); 
      saveAsItem.addActionListener(mlist);
      saveItem.addActionListener(mlist);
      exitItem.addActionListener(mlist);
      mbar.add(fileMenu);

      Menu commandsMenu= new Menu("Commands",true);
      commandsMenu.add(unloadItem= new MenuItem("Unload"));
      unloadItem.addActionListener(mlist);
      commandsMenu.add(buildPieceItem= new MenuItem("Build Piece"));
      buildPieceItem.addActionListener(mlist);
      commandsMenu.add(getMapItem= new MenuItem("Get Map"));
      getMapItem.addActionListener(mlist);
      commandsMenu.add(endTurnItem= new MenuItem("End Turn"));
      endTurnItem.addActionListener(mlist);
      commandsMenu.add(reviewTurnItem= new MenuItem("Review Turn"));
      reviewTurnItem.addActionListener(mlist);
      commandsMenu.add(productionSelectModeItem= new MenuItem("Production Select Mode"));
      productionSelectModeItem.addActionListener(mlist);
      commandsMenu.add(pieceSelectModeItem= new MenuItem("Piece Select Mode"));
      pieceSelectModeItem.addActionListener(mlist);
      mbar.add(commandsMenu);

      Menu serverCommandsMenu= new Menu("ServerCommands",true); 
      serverCommandsMenu.add(saveGameItem= new MenuItem("SaveGame")); 
      saveGameItem.addActionListener(mlist);
      serverCommandsMenu.add(attackerAlwaysWinsItem= new MenuItem("Attacker always wins")); 
      attackerAlwaysWinsItem.addActionListener(mlist);
      serverCommandsMenu.add(defenderAlwaysWinsItem= new MenuItem("Defender always wins")); 
      defenderAlwaysWinsItem.addActionListener(mlist);
      serverCommandsMenu.add(fairCombatItem= new MenuItem("Fair combat")); 
      fairCombatItem.addActionListener(mlist);
      mbar.add(serverCommandsMenu);

      f.setMenuBar(mbar);

      f.setLayout(new BorderLayout());
      statusBar.setBackground(Color.red);
      statusBar.setForeground(Color.white);
   }

   // get/set mehtods
   public Frame getFrame(){return f;}
   public void setStatusMessage(String msg){statusBar.setText(msg);}
   public void enableStatusBar(){f.add(statusBar,"North");}
   public void setMap(Map m){
      /* It is necessary to add the status bar after the map so that the map can receive key events,
       * hence the lines below that remove and add it after the map.
       */
      if (this.m!=null){
         f.remove(statusBar);
         f.remove(this.m);
         this.m.removeFromFrame();
      }
      this.m= m;
      m.setDisplaySize(8,4,f);
      m.setPieceCommandListener(
         new PieceCommandAdapter(){
            public void pieceCommand(Piece p, int command){
               if (command==PieceCommandListener.PIECE_ACTIVATE){
                  esc.displayPieceStatus(p);
               }
               else {
                  esc.sendProductionChangeMessage(p);
               }
            }
         }
      );
      f.add(m,"Center");
      f.add(statusBar,"North");
      f.pack();
   }

   void connectToServer(String host, String port1) throws
      java.net.UnknownHostException, java.io.IOException
   {
      // connect to remote server
      InetAddress addr= InetAddress.getByName(host);
      int port= Integer.parseInt(port1);
      Socket s= new Socket(addr,port);

      // open read and write streams to it.
      in= new BufferedReader(new InputStreamReader(s.getInputStream())); 
      out= new PrintWriter(new OutputStreamWriter(s.getOutputStream())); 

      // close previous connection, if any
      if (esc!=null) {
System.out.println("stopping server connection");
//esc.closeConnections();
         esc.stop();
      }

      // hook up EmpireServerConnection to the streams and start it
      esc= new EmpireServerConnection(this,in,out); 
      esc.start();
   }

   public void run(){
      f.addWindowListener(new WindowAdapter(){
         public void windowClosing(WindowEvent event){
            f.dispose();
            try {
               FileWriter fw= new FileWriter("client.map");
               m.write(fw,-1);
               fw.flush();
               fw.close();
            }
            catch (IOException e){}
            Globals.exit("");
         }
      });
      
      f.setSize(300,200);
      f.setVisible(true);
      System.out.println("EmpireClient started");
   }

   public static void main(String args[]) throws Exception{
//      String usage="java GEMpire.EmpireClient";
//      if (args.length<2){
//         Globals.exit(usage);
//      }

      Globals.packageStaticInit(true);

      EmpireClient ec= new EmpireClient();
      ec.start();
   }
}

