//FILE: EmpireClientConnection.java

/* GEMpire (Generic Empire for the Masses), a java package to support internet oriented 
 * empire-like strategy games and associated utilities.
 * Copyright (C) 1998 Christopher L. Marshall
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * My current email is christopherlmarshall@yahoo.com.  My home address is:
 * 10006 Leamoore Lane,
 * Vienna VA, 22181
 * 
 * It is my hope that this modest effort might assist others in producing Java
 * programs to implement a varied host of empire-like games.
 */

package GEMpire;

import java.io.*;

public class EmpireClientConnection extends Thread{
   // instance variables
   EmpireServer   esvr;

   BufferedReader in;   // messages from the EmpireClient arrive on this reader.
   PrintWriter    out;  // used to send messages to the EmpireClient's socket connection.
   PrintWriter    pout; // pout and sout store the outgoing message so that it can be logged to a 
   StringWriter   sout; // an output file as well as to the EmpireClient's socket connection.
   PrintWriter    logfile; // the file where Server to Client messages are logged.

   int playerNumber= -1;

   // constructors and init
   public EmpireClientConnection(EmpireServer esvr, BufferedReader in, PrintWriter out){ 
      super(); 
      this.esvr= esvr; 
      this.in= in; 
      this.out= out; 
      sout= new StringWriter(); 
      pout= new PrintWriter(sout);
      logfile= null; 
   } 

   // set/get
   public void setServer(EmpireServer esvr){this.esvr= esvr;} 
   public void setLogfile(PrintWriter pw){logfile= pw;} 

   // user display
   public void reportToUser(String msg){Globals.out.println(msg); Globals.out.flush();} 
 
   // login

   // this method passes the player number and password to the server for
   // verification.  It then informs the player if he was successful or not.
   boolean loginReplySuccessful(String mBody){ 
      try { 
         BufferedReader x= new BufferedReader(new StringReader(mBody)); 
         String line;

         // get player number
         line= x.readLine(); 
         if (line==null){
            sendPlayerMessage(Message.Canned.MALFORMED_LOGIN,false);
            return false;
         }
         playerNumber= Integer.parseInt(line);                

         // get password
         line= x.readLine(); 
         if (line==null){
            sendPlayerMessage(Message.Canned.MALFORMED_LOGIN,false);
            return false;
         }
         String password=line.trim();                

         // attempt to login to the server
         if (esvr.loginPlayer(this,playerNumber,password)){ 
            sendPlayerMessage(Message.Canned.LOGIN_SUCCESSFUL,false); 
            return true; 
         } 
         else { 
            sendPlayerMessage( 
               Message.Canned.LOGIN_DENIED1+
                  playerNumber+ 
                  Message.Canned.LOGIN_DENIED2,
               false 
            ); 
            sendPlayerMessage(esvr.getPlayersAsString(),false); 
         } 
      } 
      catch (NumberFormatException e){ 
         sendPlayerMessage(Message.Canned.MALFORMED_LOGIN,false); 
      } 
      catch (IOException e){ 
         Globals.exit(e.toString()); 
      } 
 
      return false; 
   } 
 
   boolean loginPlayer() throws IOException{ 
      String msg[]; 
 
      // Send list of available slots 
      sendPlayerMessage(esvr.getPlayersAsString(),false); 
 
      // Request Login 
      pout.println(Message.S2C.LOGIN_REQUEST); 
      pout.println(Message.Marker.END_OF_MESSAGE); 
      sendMessage(false); 
 
      // read reply
      msg= getMessage(in); 
 
      // attempt login with player and password
      if (msg[0].equals(Message.C2S.LOGIN_REPLY))
         if (loginReplySuccessful(msg[1])){
            sendSimpleMessage(Message.S2C.LOGIN_SUCCEEDED,false);
            return true;
         }
      else {
         sendPlayerMessage(Message.Canned.PLEASE_LOGIN_FIRST,false); 
         sendSimpleMessage(Message.S2C.LOGIN_FAILED,false);
      }

      return false;
   } 
 
   // server -> client messages

   void sendCombatReportTo(Piece p1, Piece p2, int player){ 
      pout.println(Message.S2C.COMBAT_REPORT); 
      p1.write(pout,player); 
      p2.write(pout,player); 
      pout.println(Message.Marker.END_OF_MESSAGE); 
      sendMessage(true); 
   } 
 
   void grantMoveRequest(){ 
      pout.println(Message.S2C.PIECE_MOVE_REPLY); 
      pout.println(Message.Marker.PIECE_MOVE_GRANTED); 
      pout.println(Message.Marker.END_OF_MESSAGE); 
      sendMessage(false); 
   } 
 
   void denyMoveRequest(){ 
      pout.println(Message.S2C.PIECE_MOVE_REPLY);
      pout.println(Message.Marker.PIECE_MOVE_DENIED);
      pout.println(Message.Marker.END_OF_MESSAGE); 
      sendMessage(false); 
   } 
 
   void sendActivatePieceMessage(Piece p){ 
      pout.println(Message.S2C.ACTIVATE_PIECE); 
      pout.println(p.getRow()); 
      pout.println(p.getColumn()); 
      pout.println(p.getId());  
      pout.println(Message.Marker.END_OF_MESSAGE); 
      sendMessage(false); 
   } 
 
   void sendCellUpdateMessage(String msg){ 
      if (msg!=null){ 
         pout.println(Message.S2C.CELL_UPDATE); 
         pout.println(msg); 
         pout.println(Message.Marker.END_OF_MESSAGE); 
         sendMessage(true); 
      } 
   } 
 
   void sendPlayerMessage(String msg, boolean log){ 
      pout.println(Message.S2C.PLAYER_MESSAGE); 
      pout.println(msg); 
      pout.println(Message.Marker.END_OF_MESSAGE); 
      sendMessage(log); 
   } 
 
   void sendTurnInfoMessage(boolean logToFile){ 
      pout.println(Message.S2C.TURN_INFO);
      pout.println(esvr.getCurrentTurn()); 
      pout.println(Message.Marker.END_OF_MESSAGE); 
      sendMessage(logToFile); 
   }

   // This handles the case of a message that contains a header only.
   // A simple flag type message with no other contents.
   void sendSimpleMessage(String msg, boolean logToFile){
      pout.println(msg);
      pout.println(Message.Marker.END_OF_MESSAGE); 
      sendMessage(logToFile); 
   }
 
   void sendMapMessage(boolean logToFile) throws IOException{    
      sendTurnInfoMessage(logToFile); 
 
      String ms= esvr.getMapAsString(playerNumber); 
      BufferedReader sin= new BufferedReader(new StringReader(ms)); 
 
      pout.println(Message.S2C.FULL_MAP); 
 
      // Copy map string to remote connection. 
      String line= sin.readLine(); 
      while (line!=null){ 
         pout.println(line); 
         line= sin.readLine(); 
      } 
 
      pout.println(Message.Marker.END_OF_MESSAGE); 
      sendMessage(logToFile); 
   } 
 
   // client message handlers

   void handlePieceMoveMessage(String msg) throws IOException{ 
      BufferedReader sin= new BufferedReader(new StringReader(msg)); 
      Piece p= new Piece(sin.readLine()); 
      int newRow= Integer.parseInt(sin.readLine()); 
      int newCol= Integer.parseInt(sin.readLine()); 
      if ((p=esvr.movePieceTo(playerNumber,p,newRow,newCol))!=null){ 
         grantMoveRequest(); 
         sendActivatePieceMessage(p); 
      } 
      else 
         denyMoveRequest(); 
   } 
 
   void handleReviewTurnMessage(String mBody) throws IOException, FileNotFoundException{ 
      // Get requested turn number. 
      int turn; 
      try {turn= Integer.parseInt(mBody);} 
      catch (NumberFormatException e){ 
         sendPlayerMessage("Could not read turn number requested",false); 
         return; 
      } 
      if (turn>esvr.getCurrentTurn()||turn<1){ 
         sendPlayerMessage("Not a valid turn!",false); 
         return; 
      } 
      pout.println(Message.S2C.TURN_REVIEW); 
      pout.println(Message.Marker.END_OF_MESSAGE); 
      sendMessage(false); 
 
      // Copy log of turn to EmpireServerConnection. 
      BufferedReader turnFile= new BufferedReader( 
         new FileReader( 
            esvr.getFilenameForTurn(turn,playerNumber) 
         ) 
      ); 
      String line= turnFile.readLine(); 
      while (line!=null){ 
         out.println(line); 
         line= turnFile.readLine(); 
      } 
 
      pout.println(Message.Marker.TURN_REVIEW_END); 
      pout.println(Message.Marker.END_OF_MESSAGE); 
      sendMessage(false); 
 
   } 
 
   // main message methods
 
   /** This method is necessary because we require that a log of messages sent from the EmpireServer 
     * to the EmpireClient be kept. 
     */ 
   void sendMessage(boolean logToFile){ 
      // Send message to EmpireClient's socket connection (actually read by an EmpireServerConnection). 
      pout.flush(); 
      out.print(sout.toString()); out.flush(); 
 
      // Print the message to the log file. 
      if (logfile!=null&&logToFile){logfile.print(sout.toString()); logfile.flush();} 
 
      // close writers
      try {
         pout.close(); 
         sout.close(); 
      } catch (IOException e){
         Globals.reportError("couldn't close StringWriter or PrintWriter used to write to client connection");
      } 
 
      // Get new StringWriter and PrintWriter for next message. 
      sout= new StringWriter(); 
      pout= new PrintWriter(sout); 
   } 
 
   String[] getMessage(BufferedReader b_in) throws IOException{ 
      String msg[]= new String[2]; 
      StringWriter sw= new StringWriter(); 
      PrintWriter pw= new PrintWriter(sw); 
      msg[0]= b_in.readLine(); 
 
      String line= b_in.readLine();
      if (line==null) return null;

      while (!line.equals(Message.Marker.END_OF_MESSAGE)){ 
         if (!line.trim().equals("")){ 
            pw.println(line); 
         } 
         line= b_in.readLine(); 
      } 
 
      pw.flush(); 
      msg[1]= sw.toString().trim(); 
 
      return msg; 
   } 
 
   void processMessage(String mHeader, String mBody) throws IOException{
      if (mHeader.equals(Message.C2S.GET_MAP)){
         sendMapMessage(false);
      }
      else if (mHeader.equals(Message.C2S.PIECE_MOVE_REQUEST)){
         handlePieceMoveMessage(mBody);
      }
      else if (mHeader.equals(Message.C2S.END_TURN)){
         esvr.endTurn(playerNumber);
      }
      else if (mHeader.equals(Message.C2S.REVIEW_TURN)){
         Globals.debugPrintln("ReviewTurn");
         try {handleReviewTurnMessage(mBody);}
         catch (FileNotFoundException  e){
            Globals.debugPrintln(
               "EmpireClientConnection.processMessage(): Could not open log file: "+e
            );
            throw new IOException();
         }
         catch (IOException e){
            Globals.debugPrintln(
               "EmpireClientConnection.processMessage(): Could not read log file: "+e
            );
            throw new IOException();
         }
      }
      else if (mHeader.equals(Message.C2S.SET_PASSWORD)){
         esvr.setPassword(playerNumber,mBody); 
      } 
      else if (mHeader.equals(Message.C2S.PRODUCTION_CHANGE)){
         esvr.changeProduction(playerNumber,new Piece(mBody)); 
      } 
      else if (mHeader.equals(Message.C2S.BUILD_PIECE)){
         esvr.buildPiece(playerNumber,new Piece(mBody));
      }
      else if (mHeader.equals(Message.C2S.SET_ATTACKER_ALWAYS_WINS)){
         if (playerNumber==0)
            Globals.setAttackerAlwaysWins(); 
         else
            sendPlayerMessage("Permission denied",false);
      }
      else if (mHeader.equals(Message.C2S.SET_DEFENDER_ALWAYS_WINS)){
         if (playerNumber==0)
            Globals.setDefenderAlwaysWins(); 
         else
            sendPlayerMessage("Permission denied",false);
      }
      else if (mHeader.equals(Message.C2S.SET_COMBAT_FAIR)){
         if (playerNumber==0)
            Globals.setCombatFair();
         else
            sendPlayerMessage("Permission denied",false);
      }
      else if (mHeader.equals(Message.C2S.SAVE_GAME)){
         if (playerNumber==0)
            esvr.saveGame(mBody);
         else
            sendPlayerMessage("Permission denied",false);
      }
      else {
         reportToUser("unknown message:");
         reportToUser(mHeader);
         reportToUser(mBody);
      }
   }

   public void run(){
      String msg[];
      try {
         if (loginPlayer()){
            reportToUser("Player "+playerNumber+" logged in.");
            while ((msg=getMessage(in))!=null){
               processMessage(msg[0],msg[1]);
            }
         }
      } catch (IOException e){}
      finally{
         reportToUser("Player "+playerNumber+" logged off.");
         esvr.logoffPlayer(playerNumber);
      }
   }
}

