//FILE: EmpireServer.java

/* GEMpire (Generic Empire for the Masses), a java package to support internet oriented 
 * empire-like strategy games and associated utilities.
 * Copyright (C) 1998 Christopher L. Marshall
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * My current email is christopherlmarshall@yahoo.com.  My home address is:
 * 10006 Leamoore Lane,
 * Vienna VA, 22181
 * 
 * It is my hope that this modest effort might assist others in producing Java
 * programs to implement a varied host of empire-like games.
 */

package GEMpire;

import java.util.*;
import java.awt.*;
import java.io.*;
import java.net.*;
import java.awt.event.*;

public class EmpireServer extends Thread{
   class MenuItemListener implements ActionListener{ 
      public void actionPerformed(ActionEvent event){ 
         MenuItem item= (MenuItem)event.getSource(); 
         if (item==exitItem){ 
            mframe.dispose(); 
            Globals.exit(""); 
         } 
         else if (item==newItem){ 
            Globals.out.println("newItem"); Globals.out.flush(); 
         } 
         else if (item==saveItem){ 
            Globals.out.println("saveItem"); Globals.out.flush(); 
         } 
         else if (item==saveAsItem){ 
            Globals.out.println("saveAsItem"); Globals.out.flush(); 
            String mapFile= IOUtil.getFilename(FileDialog.SAVE,mframe); 
            if (mapFile!=null) 
               saveGame(mapFile); 
         } 
         else if (item==openItem){ 
            String mapFile= IOUtil.getFilename(FileDialog.LOAD,mframe); 
            if (mapFile!=null) 
               loadGame(mapFile); 
            Globals.out.println("openItem"); Globals.out.flush(); 
         } 
         else if (item==cellDumpItem){ 
            Globals.out.println("cellDumpItem"); Globals.out.flush(); 
         } 
         else if (item==attackerAlwaysWinsItem){ 
            Globals.setAttackerAlwaysWins(); 
            Globals.out.println("set Attacker Always Wins"); Globals.out.flush(); 
         } 
         else if (item==defenderAlwaysWinsItem){ 
            Globals.setDefenderAlwaysWins(); 
            Globals.out.println("set Defender Always Wins"); Globals.out.flush(); 
         } 
         else if (item==fairCombatItem){ 
            Globals.setCombatFair(); 
            Globals.out.println("set Combat Fair");  Globals.out.flush(); 
         } 
         else if (item==newTurnItem){ 
            Globals.out.println("refreshing pieces"); Globals.out.flush(); 
            implementNewTurn(); 
         } 
      } 
   } 
 
   static int defaultNumDisplayRows= 10;
   static int defaultNumDisplayCols= 4;

   private boolean noGUI;
   private Frame mframe= null;
   private Map m;
   private Game game;
   private boolean players[];

   private EmpireConnectionAcceptor eca;
   private EmpireClientConnection clients[];

   private MenuBar mbar;
   MenuItem newItem,openItem,saveAsItem,saveItem,exitItem,cellDumpItem,newTurnItem;
   MenuItem attackerAlwaysWinsItem,defenderAlwaysWinsItem,fairCombatItem;

   // constructors and initialization

   public EmpireServer(int port, String mapFile, boolean noGUI) throws IOException{
      super();
      this.noGUI= noGUI;
      if (!noGUI)
         mframe= new MainFrame("Empire Server");
      loadGame(mapFile);
      eca= new EmpireSocketConnectionAcceptor(port);
      eca.setServer(this);
      if (!noGUI)
         initMenuBar();
   }

   public EmpireServer(BufferedReader in,PrintWriter out, String mapFile, boolean noGUI) 
      throws BadRowAndColumnValuesException, InterruptedException
   {
      super();
      this.noGUI= noGUI;
      if (!noGUI)
         mframe= new MainFrame("Empire Server");
      loadGame(mapFile);
      eca= new EmpireConnectionAcceptor(in,out);
      eca.setServer(this);
      if (!noGUI)
         initMenuBar();
   }

   private void initMenuBar(){
      MenuItemListener mlist= new MenuItemListener();
      mbar= new MenuBar();

      Menu fileMenu= new Menu("File",true);
      fileMenu.add(newItem=    new MenuItem("New"));
      fileMenu.add(openItem=   new MenuItem("Open"));
      fileMenu.add(saveAsItem= new MenuItem("Save As"));
      fileMenu.add(saveItem=   new MenuItem("Save"));
      fileMenu.addSeparator();
      fileMenu.add(exitItem=   new MenuItem("Exit"));
      newItem.addActionListener(mlist);
      openItem.addActionListener(mlist);
      saveAsItem.addActionListener(mlist);
      saveItem.addActionListener(mlist);
      exitItem.addActionListener(mlist);
 
      Menu commandMenu= new Menu("Command",true);
      commandMenu.add(newTurnItem= new MenuItem("New Turn"));
      newTurnItem.addActionListener(mlist);

      Menu debugMenu= new Menu("Debug",true);
      debugMenu.add(cellDumpItem= new MenuItem("Cell Dump"));
      debugMenu.add(attackerAlwaysWinsItem= new MenuItem("Attacker Always Wins"));
      debugMenu.add(defenderAlwaysWinsItem= new MenuItem("Defender Always Wins"));
      debugMenu.add(fairCombatItem= new MenuItem("Fair Combat"));
      cellDumpItem.addActionListener(mlist);
      attackerAlwaysWinsItem.addActionListener(mlist);
      defenderAlwaysWinsItem.addActionListener(mlist);
      fairCombatItem.addActionListener(mlist);

      mbar.add(fileMenu);
      mbar.add(commandMenu);
      mbar.add(debugMenu);

      mframe.setMenuBar(mbar);
   }


   // info retrieval 
   public int getCurrentTurn(){return game.getCurrentTurn();} 

   public String getFilenameForTurn(int turn, int player){ 
      return game.getFilenameForTurn(turn,player); 
   }

   public boolean[] getPlayers(){return players;}

   public String getPlayersAsString(){
      StringBuffer sb= new StringBuffer();
      sb.append("Valid players are: ");
      for (int i=0; i<players.length; i++) if (players[i]) sb.append(i+" ");
      return sb.toString();
   }

   public synchronized String getMapAsString(int player){ 
      StringWriter sw= new StringWriter();
      m.write(sw,player); 
      return sw.toString(); 
   } 
 
   // map/piece altering methods
   public synchronized void endTurn(int i){
      if (game.endTurn(i)){
         if (game.everyoneHasFinishedTurn())
            implementNewTurn();
         int tmp= game.getControllingPlayer();
         if (tmp!=-1){
            clients[tmp].sendPlayerMessage("Your are in control of the board!",false);
            broadcastMessageToPlayers("Player "+tmp+" just took control of the board",false);
         }
      }
   }

   public synchronized void implementNewTurn(){ 
      Vector deadPieces; 
      deadPieces= m.implementNewTurn(); 
      Enumeration enum= deadPieces.elements(); 
      while (enum.hasMoreElements()){ 
         Piece p= (Piece)(enum.nextElement()); 
         int pOwner= p.getOwner(); 
System.out.println("piece crashed: "+p); 
         clients[pOwner].sendCellUpdateMessage( 
            m.writeCellUpdateTo(p.getRow(),p.getColumn(),pOwner,false) 
         ); 
         clients[pOwner].sendPlayerMessage("Your piece "+p+" ran out of fuel and crashed",true); 
      } 

      // figure out if any players were killed off
      boolean newPlayers[]= new boolean[players.length];
      newPlayers= m.findAllPlayers();
      for (int i=0; i<players.length; i++){
         if (players[i]&&!newPlayers[i]){
            broadcastMessageToPlayers(
               "player "+i+" was annihilated on turn"+game.getCurrentTurn(),
               true
            );
            clients[i].sendPlayerMessage(
               "that was YOU buddy!  Better luck next time :-(",
               true
            );
            game.killPlayer(i);
         }
      }
      players= newPlayers;

      game.implementNewTurn(); 
 
      // Advise client connections of new log files. 
      for (int i=0; i<clients.length; i++){ 
         if (game.validPlayer(i)){ 
            clients[i].setLogfile(game.getLogfile(i)); 
            try {clients[i].sendMapMessage(true);} 
            catch (IOException e){ 
               Globals.exit( 
                  "EmpireServer.implementNewTurn(): couldn't send map message to player "+ 
                  i+" :"+e 
               ); 
            } 
         } 
      } 
 
      broadcastMessageToPlayers("New Turn! It is now turn: "+game.getCurrentTurn(),false); 
   } 
 
   public void changeProduction(int player, Piece p){
      int pid= p.getOwner();
      if (player!=pid){
         clients[player].sendPlayerMessage(
            "You can't order player "+pid+"'s piece!  You are player "+player+"!",
            false
         );
         return;
      }
      Piece p2= m.getPieceAt(p.getRow(),p.getColumn());
      if (p2==null){
         clients[player].sendPlayerMessage(
            "You requested to change production for a piece that doesn't exist!",
            false
         );
      }
      else {
         if (p2.setProduction(p.getProduction())){
            clients[player].sendPlayerMessage("Production changed",false);
         }
         else {
            clients[player].sendPlayerMessage("Production change failed",false);
         }
      }
   }

   public void buildPiece(int player, Piece p){
      int pid= p.getOwner();
      if (player!=pid){
         clients[player].sendPlayerMessage(
            "You can't order player "+pid+"'s piece!  You are player "+player+"!",
            false
         );
         return;
      }
      // The following line has the effect of preventing build orders being accepted by
      // a piece that is being carried by another piece.  The piece must be occupying a
      // map cell and not another piece if it is to receive build orders, 
      // since Map.getPiece(int row, int col) returns the piece occupying row,col, and not
      // any of the pieces carried by that piece.
      Piece p2= m.getPieceAt(p.getRow(),p.getColumn());
      if (p2==null){
         clients[player].sendPlayerMessage(
            "You requested to change build orders for a piece that doesn't exist!",
            false
         );
      }
      else {
         if (p2.setBuildType(p.getBuildType())){
            clients[player].sendPlayerMessage("Build type changed",false);
         }
         else {
            clients[player].sendPlayerMessage("Build change failed",false);
         }
      }
   }

   public synchronized Piece movePieceTo(int playerNumber, Piece p, int row, int col){
      if (!game.controlsBoard(playerNumber)){
         clients[playerNumber].sendPlayerMessage(
            "It is not your turn yet! Player "+
               game.getControllingPlayer()+
               " controls the board.",
            false
         );
         return null;
      }
      if (!m.inBounds(row,col)) return null;

      int old_rc[]= p.getRowAndColumn();
      // The line below gets the Piece object from the server Map with the same id, row, and column 
      // as the referece 'p' passed in. The p reference passed
      // in to this method refers to an object constructed from a text description sent from the client.
      // It is important to not attach this object to the map because then any other server side references
      // to that piece would be left dangling.  The newly constructed piece is used to convey only the
      // row, column, and id of the piece.  The reference to the server side object representing that
      // piece is then fetched by the following line.
      p= m.getPieceAt(p.getId(),old_rc[0],old_rc[1]);
      if (p==null) 
         Globals.exit("EmpireServer.movePieceTo(): could not retrieve piece from map "+p);

      if (playerNumber!=p.getOwner()) return null;

      MovementReport mreport= m.getPieceMover().movePieceTo(p,row,col,players);
      if (mreport.moveOccurred()){
         // Send player moving piece update of cells surrounding newly occupied cell, 
         // if target cell was actually reached or captured.
         if (playerNumber==m.getOwner(row,col)){
            for (int i=0; i<MapHex.numNeighbors(); i++){
               int rc[]= MapHex.getNeighborCoordinates(i,row,col);
               clients[playerNumber].sendCellUpdateMessage(
                  m.writeCellUpdateTo(rc[0],rc[1],playerNumber,false)
               );
            }
         }

         // Issue combat reports to combatants.
         if (mreport.combatOccurred()){
            Piece p1= mreport.getReportP1();
            Piece p2= mreport.getReportP2();
            int defender= mreport.getDefender();

            clients[p.getOwner()].sendCombatReportTo(p1,p2,p.getOwner());
            if (defender!=-1)
               clients[defender].sendCombatReportTo(p1,p2,defender);          
         }

         // Notify all players that could see affected cells of results
         notifyAllOfCellUpdate(
            row,
            col,
            mreport.getDestinationVisibilityBeforeMove(),
            mreport.getDestinationVisibilityAfterMove(),
            mreport.shouldAssumeVisibleTo()
         );
         notifyAllOfCellUpdate(
            old_rc[0],
            old_rc[1],
            mreport.getSourceVisibilityBeforeMove(),
            mreport.getSourceVisibilityAfterMove(),
            mreport.shouldAssumeVisibleTo()
         );

         return p;
      }
      return null;
   }

   // player communication methods

   private void notifyAllOfCellUpdate( 
      int       row,  
      int       col, 
      boolean[] wasVisibleTo, 
      boolean[] isVisibleTo, 
      int       assumeVisibleTo 
   ){ 
      for (int i=0; i<players.length; i++){ 
         if (game.validPlayer(i)){ 
            if ((wasVisibleTo[i]&&!isVisibleTo[i])||isVisibleTo[i]||(assumeVisibleTo==i)||
               i==0) // player 0 gets every cell update
            { 
               clients[i].sendCellUpdateMessage( 
                  m.writeCellUpdateTo( 
                     row,  
                     col,  
                     i, 
                     (i==assumeVisibleTo) 
                  ) 
               ); 
            } 
         } 
      } 
   } 

   void broadcastMessageToPlayers(String msg, boolean logIt){ 
      for (int i=0; i<clients.length; i++){ 
         if (game.validPlayer(i)){ 
            clients[i].sendPlayerMessage(msg,logIt); 
         } 
      } 
   } 
 
   public synchronized void addConnection(EmpireClientConnection ecc){ 
      ecc.setServer(this); 
      ecc.start(); 
   } 
 
   public synchronized boolean loginPlayer(EmpireClientConnection ecc, int i, String password){
//     if (!game.validatePassword(i,password)) return false;
      if (clients[i] instanceof EmpireNullClientConnection) if (game.loginPlayer(i,password)){
         StringWriter sw= new StringWriter();
         PrintWriter pw= new PrintWriter(sw);

         // we don't want game to login player 0 because the referee
         // should never get control of the board
//         if (i!=0) game.loginPlayer(i);

         clients[i]= ecc;
         clients[i].setLogfile(game.getLogfile(i));

         // No need to bother with any of this if the referee (player 0) is logging in.
         // Otherwise, send a greeting with current game info to the new player.
         if (i!=0){
            // let everyone know who just logged in
            broadcastMessageToPlayers("Player "+i+" just logged in",false);

            // Send greeting with game info about who is logged in and such.
            if (game.getControllingPlayer()==i)
               pw.println("You are in control of the board!");
            else
               pw.println("Player "+game.getControllingPlayer()+" is in control of the board");

            pw.print("Other players presently logged in:");
            for (int j=0; j<clients.length; j++){
               if (game.isLoggedIn(j)) pw.println(" "+j);
            }
            pw.println("Current Turn: "+game.getCurrentTurn());
            pw.flush();

            ecc.sendPlayerMessage(sw.toString(),false);
         }

         return true;
      }
      return false;
   }

   public synchronized void logoffPlayer(int i){
      if (game.validPlayer(i))
{
         clients[i]= new EmpireNullClientConnection(this,game.getLogfile(i),i);
         game.logoffPlayer(i);
         broadcastMessageToPlayers("Player "+i+" just logged off",false);
      }
      else 
         clients[i]= null;
   }

   // game related

   public void setPassword(int playerNumber, String password){
      game.setPassword(playerNumber,password);
   }
   private void loadGame(String mapFile){
      try {
         // Dispose of present map, if any, first
         if (m!=null){
            m.removeFromFrame();
            if (!noGUI) mframe.remove(m);
         }

         // Read new map, get list of players that have at least one piece on the map.
         m= new Map(mframe,new FileReader(mapFile));
         if (noGUI) m.setPieceMover(new PieceMover(m,false));
         players= m.getPlayers();

         // Player 0 is an interface for the game referee to login to the server
         // and control it.  So it always has to be a valid player.
         players[0]= true;

         // If mapFile has any game info, read from file.  Otherwise, initialize new game.
         try {
            game= new Game(new BufferedReader(new FileReader(mapFile)));
         }
         catch (IOException e){
            game= new Game(players);
         }
         
         // Assign a NullClient to each player number to log messages until player logs in.
         if (clients==null){
            clients= new EmpireClientConnection[players.length];
            for (int i=0; i<clients.length; i++){
               if (game.validPlayer(i))
                  clients[i]= new EmpireNullClientConnection(this,game.getLogfile(i),i);
            }
         }
         else {
/*
            // This case should only happen during debugging.
            for (int i=0; i<clients.length; i++){
               if (game.validPlayer(i)){
                  clients[i].sendMapMessage(false);
                  game.loginPlayer(i);
               }
            }
*/
         }

         // Add new map panel to frame, if operating in a GUI environment.
         if (!noGUI){
            mframe.add(m,"Center");
            m.setDisplaySize(defaultNumDisplayRows,defaultNumDisplayCols,mframe);
            mframe.pack();
         }

         if (game.isBeginning()){
            for (int i=0; i<clients.length; i++){
               if (game.validPlayer(i)){
                  clients[i].sendMapMessage(true);
               }
            }
         }
      }
      catch (FileNotFoundException e){
         Globals.exit("EmpireServer.loadGame(): could not find "+mapFile+"\n\r"+e);
      }
      catch (IOException e){ 
         Globals.exit("EmpireServer.loadGame(): could not read "+mapFile+"\n\r"+e);
      }
      catch (BadRowAndColumnValuesException e){
         Globals.exit(
            "EmpireServer.loadGame(): map in file "+mapFile+" did not have "+
            "a proper number of rows and columns"
         );
      }
   }

   public void saveGame(String mapFile){ 
      try { 
         FileWriter  fw= new FileWriter(mapFile); 
         PrintWriter pw= new PrintWriter(fw); 
         m.write(fw); 
         game.write(pw); 
         pw.close(); 
         fw.close(); 
      } 
      catch (IOException e){ 
         System.err.println("EmpireServer.saveGame(): Could not write to file!"); 
         System.err.println(e.toString()); 
      } 
   } 
 
   public void run(){ 
      if (!noGUI) mframe.setVisible(true); 
 
      Globals.out.println("EmpireServer started"); 
      eca.start(); 
//      debugLoop(); 
   } 
 
   private void debugLoop(){ 
      try { 
         BufferedReader in= new BufferedReader(new InputStreamReader(System.in)); 
         PrintWriter out= new PrintWriter(new OutputStreamWriter(System.out)); 
         String line= in.readLine(); 
          
         while (line!=null){ 
            if (line.equals("q")){ 
               out.print("row> "); out.flush(); 
               int row= Integer.parseInt(in.readLine().trim()); 
               out.print("col> "); out.flush(); 
               int col= Integer.parseInt(in.readLine().trim()); 
               m.debugReportOn(out,row,col,players); 
               out.flush(); 
            } 
            line= in.readLine(); 
         } 
      } 
      catch (Exception e){ 
         Globals.exit(e.toString()); 
      } 
   } 
 
   private void generateTestMap(){
      try {
         m= new Map(mframe,100,40,8,2);
         m.attachPiece(new Piece(0,0,0,0));
         m.setAllTerrain(TerrainTypes.SEA);
         for (int i=0; i<Piece.numTypes(); i++){
            for (int j=0; j<PieceDatabase.defaultNumPossiblePlayers; j++){
               Piece p= new Piece(i,j,j,i);
               if (!p.canMoveOver(m.getTerrain(j,i)))
                  m.setTerrain(j,i,TerrainTypes.LAND);
               m.attachPiece(new Piece(i,j,j,i));
            }
         }
         m.setDisplaySize(8,4,mframe);
      }
      catch (Exception e){
         Globals.exit("EmpireServer couldn't generate test map\n\r"+e);
      }
   }

   public static void main(String args[]) throws Exception{
      String usage="java GEMpire.EmpireServer <port> <starting map filename> <'GUI' (optional)>";
      if (args.length<2) Globals.exit(usage);

      // Determine if a GUI is desired (if the server is running on a unix box through a shell account,
      // then we have to be careful to not invoke the AWT or we will cause an exception to the effect
      // that the X11 server can't open a display.
      boolean noGUI= true;
      if (args.length>=3) if (args[2].trim().equals("GUI")) noGUI= false;

      int port= Integer.parseInt(args[0]);
      Globals.packageStaticInit(!noGUI);
      EmpireServer es= new EmpireServer(port,args[1],noGUI);
       
      try {Globals.out.println("Your address is "+InetAddress.getLocalHost()); Globals.out.flush();}
      catch (java.net.UnknownHostException e){
         Globals.out.println("Your address is "+InetAddress.getByName(null));  Globals.out.flush();
      }
      es.start();
   }
}
