// FILE: EmpireServerConnection.java

/* GEMpire (Generic Empire for the Masses), a java package to support internet oriented 
 * empire-like strategy games and associated utilities.
 * Copyright (C) 1998 Christopher L. Marshall
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * My current email is christopherlmarshall@yahoo.com.  My home address is:
 * 10006 Leamoore Lane,
 * Vienna VA, 22181
 * 
 * It is my hope that this modest effort might assist others in producing Java
 * programs to implement a varied host of empire-like games.
 */

package GEMpire;

import java.io.*;
import java.util.Vector;
import java.util.Enumeration;

public class EmpireServerConnection extends Thread implements KeyReceiver{
   // instance variables
   private EmpireClient ec;
   private Map m;
   int turn=-1;
   int playerNumber= -1;
   boolean loggedIn= false;

   // server<->client comm streams
   private BufferedReader in;
   private PrintWriter out;

   // for storing server replies to client requests
   private boolean replyWaiting= false; 
   private String replyHeader; 
   private String replyBody; 

   // constructors
   public EmpireServerConnection(EmpireClient ec, BufferedReader in, PrintWriter out){ 
      super(); 
      this.ec= ec; 
      this.in= in; 
      this.out= out; 
      ec.setStatusMessage(""); 
   } 
 
   // state info
   public boolean isLoggedIn(){return loggedIn;}

   // user menu commands
   public void unloadCommand(){ 
      Piece p= m.getActivePiece(); 
      if (p!=null){ 
         Piece tmp= p.getPiece(); 
         if (tmp!=null){ 
            m.activatePiece(tmp); 
         } 
      } 
   } 
   public void buildPieceCommand(){ 
      Piece p= m.getActivePiece(); 
      if (p!=null){ 
         int tmp= p.getBuildType(ec.getFrame()); 
         System.out.println("Stub for building piece from: "+p); 
         System.out.println("BuildType= "+tmp); 
         sendBuildPieceCommand(p); 
      } 
   } 
   public void promptNextPiece(){
      // get a list of all pieces that have moves left
      Vector pieces= new Vector();
      int numRows= m.getNumRows();
      int numCols= m.getNumCols();
      for (int r=0; r<numRows; r++){
         for (int c=0; c<numCols; c++){
            Piece p= m.getPieceAt(r,c);
            if (p!=null){
               if (p.getOwner()==playerNumber){
                  if (p.getMovesLeft()>0)
                     pieces.add(p);
               }
            }
         }
      }

      // activate the first one, if any
      if (pieces.size()>0){
         Piece p= (Piece)pieces.elementAt(0);
         m.activatePiece(p);
      }
   }
 
   // KeyReceiver interface method
   public void receiveKeyText(String text){ 
      if (text.equals("U")){ 
         unloadCommand(); 
      } 
      else if (text.equals("P")){
         promptNextPiece();
      }
      else System.out.println("key text=|"+text+"|");
   } 
 
   // reply mechanism
   void clearReply(){replyHeader= null; replyBody= null; replyWaiting= false;} 
   boolean replyWaiting(){return replyWaiting;} 
   String getReplyHeader(){return replyHeader;} 
   String getReplyBody(){return replyBody;} 
   synchronized void storeReply(String header, String body){ 
      if (replyWaiting()){ 
         Globals.exit( 
            "EmpireServerConnection.storeReply():  "+ 
            "previous reply has not cleared.  This shouldn't happen." 
         ); 
      } 
      replyHeader= header; 
      replyBody= body; 
      replyWaiting= true; 
      notifyAll(); 
   } 
 
   // requests to server
   public void requestMap(){ 
      out.println(Message.C2S.GET_MAP);
      out.println(Message.Marker.END_OF_MESSAGE);
      out.flush(); 
   } 
   public void sendSaveGameMessage(String filename){ 
      out.println(Message.C2S.SAVE_GAME); 
      out.println(filename);
      out.println(Message.Marker.END_OF_MESSAGE); 
      out.flush();    
   } 
   public void sendSetPasswordMessage(String password){ 
      out.println(Message.C2S.SET_PASSWORD); 
      out.println(password); 
      out.println(Message.Marker.END_OF_MESSAGE); 
      out.flush();    
   } 
   public void sendBuildPieceCommand(Piece p){ 
      out.println(Message.C2S.BUILD_PIECE); 
      p.write(out); 
      out.println(Message.Marker.END_OF_MESSAGE); 
      out.flush();    
   } 
   public void sendProductionChangeMessage(Piece p){ 
      out.println(Message.C2S.PRODUCTION_CHANGE); 
      p.write(out); 
      out.println(Message.Marker.END_OF_MESSAGE); 
      out.flush(); 
   } 
   public void sendEndTurnMessage(){ 
      out.println(Message.C2S.END_TURN); 
      out.println(Message.Marker.END_OF_MESSAGE); 
      out.flush(); 
   } 
   public void sendAttackerAlwaysWinsMessage(){ 
      out.println(Message.C2S.SET_ATTACKER_ALWAYS_WINS); 
      out.println(Message.Marker.END_OF_MESSAGE); 
      out.flush(); 
   } 
   public void sendDefenderAlwaysWinsMessage(){ 
      out.println(Message.C2S.SET_DEFENDER_ALWAYS_WINS); 
      out.println(Message.Marker.END_OF_MESSAGE); 
      out.flush(); 
   } 
   public void sendFairCombatMessage(){ 
      out.println(Message.C2S.SET_COMBAT_FAIR); 
      out.println(Message.Marker.END_OF_MESSAGE); 
      out.flush(); 
   } 
   public void sendSaveGameMessage(){ 
      out.println(Message.C2S.SAVE_GAME); 
      out.println(Message.Marker.END_OF_MESSAGE); 
      out.flush(); 
   } 
   public void sendReviewTurnMessage(){ 
      out.println(Message.C2S.REVIEW_TURN); 
      out.println( 
         UserPrompt.getString( 
            ec.getFrame(), 
            "Review Turn Dialog", 
            "Turn Number", 
             ""+turn, 
            3 
         ).trim() 
      ); 
      out.println(Message.Marker.END_OF_MESSAGE); 
      out.flush(); 
   } 
   public synchronized boolean requestPieceMove(Piece p, int row, int column){ 
      m.deactivatePiece(); 
 
      if (replyWaiting()){ 
         Globals.exit( 
            "EmpireServerConnection.requestPieceMove(): "+ 
            "a reply was waiting before one was expected, this shouldn't happen." 
         ); 
      } 
 
      out.println(Message.C2S.PIECE_MOVE_REQUEST); 
      p.write(out,-1); 
      out.println(row); 
      out.println(column); 
      out.println(Message.Marker.END_OF_MESSAGE); 
      out.flush(); 
 
      try {while (!replyWaiting()) wait();}  
      catch (InterruptedException e){throw new Error(e.toString());} 
 
      if (!getReplyHeader().equals(Message.S2C.PIECE_MOVE_REPLY)){ 
         Globals.exit( 
            "EmpireServerConnection.requestPieceMove(): "+ 
            "wrong reply received from server" 
         ); 
      } 
 
      String rbody= getReplyBody(); 
      clearReply(); 
 
      if (rbody.equals(Message.Marker.PIECE_MOVE_GRANTED)){ 
         return true; 
      } 
      else if (rbody.equals(Message.Marker.PIECE_MOVE_DENIED)){ 
         reportToUser("Piece move denied by server"); 
         return false; 
      } 
      else { 
         Globals.exit( 
            "EmpireServerConnection.requestPieceMove():"+ 
            " received invalid response: \n\r|"+rbody+"|" 
         ); 
      } 
      return false; 
   } 
 
   // user display
   public void reportToUser(String msg){System.out.println(msg);} 
   public void displayPieceStatus(Piece p){ 
      ec.setStatusMessage("Turn "+turn+" "+p.getStatus()); 
System.out.println(""+p); 
   } 
 
   // server message handlers

   void replyToLoginRequest(){ 
      // get player and password to log in as
      String fieldNames[]={"player","password"};
      String initialValues[]={"",""};
      int sizes[]={15,15};
      String x[]= UserPrompt.getStrings( 
         ec.getFrame(), 
         "Login",
         fieldNames, 
         initialValues, 
         sizes
      );
      out.println(Message.C2S.LOGIN_REPLY); 
      out.println(x[0]);
      out.println(x[1]);
      out.println(Message.Marker.END_OF_MESSAGE); 
      out.flush();

      try {
         playerNumber= Integer.parseInt(x[0]);
      }
      catch (NumberFormatException e){
         playerNumber= -1;
      }
   } 

   void handleActivatePieceMessage(String msg){ 
      try { 
         BufferedReader br= new BufferedReader(new StringReader(msg)); 
         int row= Integer.parseInt(br.readLine()); 
         int col= Integer.parseInt(br.readLine()); 
         int id= Integer.parseInt(br.readLine()); 
         if (!m.activatePiece(id,row,col)){ 
            reportToUser("Activation of Piece failed"); 
         } 
         Piece p= m.getActivePiece(); 
         if (p!=null){ 
            displayPieceStatus(p); 
         } 
      } 
      catch (IOException e){ 
         Globals.exit( 
            "could not parse ActivatePieceMessage: "+msg+"\n\r"+e.toString() 
         ); 
      } 
   } 

   void handleCombatReport(String msg){ 
      try { 
         int rca[]= null, rcb[]= null; 
 
         BufferedReader br= new BufferedReader(new StringReader(msg)); 
         Piece p1= new Piece(br); 
         Piece p2= new Piece(br); 
 
         m.drawPiece(p1); 
         m.drawPiece(p2); 
         sleep(500); 
 
for (int i=0; i<2; i++){ 
         rcb= p2.getRowAndColumn(); 
         m.drawPattern(rcb[0],rcb[1],MapHex.RED_PATTERN); 
         sleep(500); 
 
         m.drawPiece(p2); 
         sleep(500); 
 
         rca= p1.getRowAndColumn(); 
         m.drawPattern(rca[0],rca[1],MapHex.RED_PATTERN); 
         sleep(500); 
 
         m.drawPiece(p1); 
         sleep(500); 
} 
 
         m.drawCell(rca[0],rca[1]); 
         m.drawCell(rcb[0],rcb[1]); 
      } 
      catch (IOException e){ 
         Globals.exit( 
            "EmpireServerConnection.handleCombatReport(): could not parse CombatReport: "+ 
            msg+"\n\r"+e.toString() 
         ); 
      } 
      catch (InterruptedException e){ 
      } 
   } 

   void handleTurnReview()  
      throws InterruptedException, IOException, FileNotFoundException, BadRowAndColumnValuesException 
   { 
      String msg[]= getMessage(); 
      Vector messages= new Vector(); 
      boolean done= false; 
 
      while (!msg[0].equals(Message.Marker.TURN_REVIEW_END)){ 
         messages.addElement(msg); 
         msg= getMessage(); 
      } 
      Enumeration e= messages.elements(); 
      while (e.hasMoreElements()){ 
         msg= (String[]) e.nextElement(); 
         processMessage(msg[0],msg[1]); 
         sleep(100); 
      } 
   } 
 
   // main message handling

   String[] getMessage() throws IOException{
      String msg[]= new String[2];
      StringWriter sw= new StringWriter();
      PrintWriter pw= new PrintWriter(sw);
      msg[0]= in.readLine();

      String line= in.readLine();      
      while (!line.equals(Message.Marker.END_OF_MESSAGE)){
         if (!line.trim().equals("")){
            pw.println(line);
         }
         line= in.readLine();
      }

      pw.flush();
      msg[1]= sw.toString().trim();
      return msg;
   }

   boolean processMessage(String mHeader, String mBody) throws 
      FileNotFoundException, BadRowAndColumnValuesException, IOException, InterruptedException
   {
      if (mHeader.equals(Message.S2C.FULL_MAP)){
         m=new Map(null, new StringReader(mBody));
         m.setPieceMover(new RemotePieceMover(m,this));
         m.setKeyReceiver(this);
         ec.setMap(m);
         ec.setStatusMessage("Turn "+turn);
      }
      else if (mHeader.equals(Message.S2C.TURN_INFO)){
         turn= Integer.parseInt(mBody.trim());
      }
      else if (mHeader.equals(Message.S2C.PIECE_MOVE_REPLY)){ 
         storeReply(mHeader,mBody);
      }
      else if (mHeader.equals(Message.S2C.LOGIN_REQUEST)){ 
         replyToLoginRequest(); 
      } 
      else if (mHeader.equals(Message.S2C.LOGIN_SUCCEEDED)){ 
         loggedIn= true; 
         requestMap();
      }
      else if (mHeader.equals(Message.S2C.LOGIN_FAILED)){ 
         return false; 
      } 
      else if (mHeader.equals(Message.S2C.PLAYER_MESSAGE)){ 
         reportToUser("Message from Server:");
         reportToUser(mBody+"\n\r");
      }
      else if (mHeader.equals(Message.S2C.CELL_UPDATE)){ 
         m.readCellUpdate(mBody);
      }
      else if (mHeader.equals(Message.S2C.COMBAT_REPORT)){ 
         handleCombatReport(mBody);
      }
      else if (mHeader.equals(Message.S2C.ACTIVATE_PIECE)){ 
         handleActivatePieceMessage(mBody);
      }
      else if (mHeader.equals(Message.S2C.TURN_REVIEW)){ 
         handleTurnReview();
      }
      else {
         reportToUser("Unknown messsage type");
         reportToUser(mHeader);
         reportToUser(mBody);
      }

      return true;
   }

   public void closeConnections(){
      try {in.close();} catch (IOException e){}
      out.close();
   }

   public void run(){
      String msg[];
      try {
         do {
            msg= getMessage(); 
         } while (processMessage(msg[0],msg[1]));
      }
      catch (FileNotFoundException e){}
      catch (BadRowAndColumnValuesException e){}
      catch (IOException e){}
      catch (InterruptedException e){}
      finally {
         closeConnections();
System.out.println("thread stopped");
      }
   }
}

