//FILE: GraphicsUtil.java

/* GEMpire (Generic Empire for the Masses), a java package to support internet oriented 
 * empire-like strategy games and associated utilities.
 * Copyright (C) 1998 Christopher L. Marshall
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * My current email is christopherlmarshall@yahoo.com.  My home address is:
 * 10006 Leamoore Lane,
 * Vienna VA, 22181
 * 
 * It is my hope that this modest effort might assist others in producing Java
 * programs to implement a varied host of empire-like games.
 */

package GEMpire;

import java.awt.*;
import java.awt.image.*;

public class GraphicsUtil{
   public static Image getBlankImage(int w, int h){
      int pix[]= new int[w*h];
      for (int i=0; i<w*h; i++) pix[i]=0;
      return getImage(pix,w,h);
   }
   public static Image getImage(int pix[], int w, int h){
      return Toolkit.getDefaultToolkit().createImage(
         new MemoryImageSource(w, h, pix, 0, w)
      );
   }

   /** Load an image from a file (GIF or other supported type)
     */
   public static Image getImage(String file) throws InterruptedException{
      Image im= Toolkit.getDefaultToolkit().getImage(file);

      // wait for image to load.
      MediaTracker tracker= new MediaTracker(new Frame());
      tracker.addImage(im,0);
      tracker.waitForID(0);

      // get width and height
      Frame f= new Frame();
      int w= im.getWidth(f);
      int h= im.getHeight(f);
      f.dispose();

      return im;
   }

   /** Substitute one color value for another.
     */
   public static void changeColor(int x[], int from, int to){
      for (int i=0; i<x.length; i++){
         if (x[i]==from) x[i]= to;
      }
   }

   /** Substitute one color value for another.
     */
   public static Image changeColor(Image im, int from, int to, Component c){
      int pix[]= getPixels(im);
      int w= im.getWidth(c);
      int h= im.getHeight(c);
      changeColor(pix,from,to);
      return getImage(pix,w,h);
   }

   /** Extract ARGB values from a pixel value to an array of int.
     */
   public static int[] parsePixel(int pixelValue){
      int values[]= new int[4];
      values[0]= (pixelValue>>24)&0xff;
      values[1]= (pixelValue>>16)&0xff;
      values[2]= (pixelValue>> 8)&0xff;
      values[3]=  pixelValue     &0xff;
      return values;
   }

   /** Create pixel value from array of ARGB values.
     */
   public static int assemblePixel(int v[]){
      return (v[0]<<24)|
             (v[1]<<16)|
             (v[2]<< 8)|
             (v[3]    );
   }

   /** Returns an array of pixel values from an image.
     */
   public static int[] getPixels(Image im){
        Frame f= new Frame();
        int w= im.getWidth(f);
        int h= im.getHeight(f);
        f.dispose();

	int[] pixels = new int[w * h];
	PixelGrabber pg = new PixelGrabber(im, 0, 0, w, h, pixels, 0, w);
	try {
	    pg.grabPixels();
	} catch (InterruptedException e) {
	    Globals.exit("interrupted waiting for pixels!");
	}
      return pixels;
   }

/*
   public static Image makeSolidHex(int color, int rowWidth[]){
      int w = rowWidth[rowWidth.length-1];
      int h = 2*rowWidth.length;
      int pix[] = new int[w * h];
      int index = 0;
      int transparentColor= 0;
      for (int row=0; row<rowWidth.length; row++){
         int leftSpace= (w-rowWidth[row])/2;
         int col;
         // fill left and right width space.
         for (col=0; col<leftSpace; col++){
            // top of hex
            pix[row*w+col]= transparentColor;
            pix[row*w+(w-1-col)]= transparentColor;
            // bottom of hex 
            pix[(h-1-row)*w+col]= transparentColor;
            pix[(h-1-row)*w+(w-1-col)]= transparentColor;
         }
         // fill middle with color.
         while (col<leftSpace+rowWidth[row]){
            // top of hex
            pix[row*w+col]= color;
            // bottom of hex
            pix[(h-1-row)*w+col]= color;
            col++;
         }
      }

      return Toolkit.getDefaultToolkit().createImage(
         new MemoryImageSource(w, h, pix, 0, w)
      );   
   }
*/

   /** Makes a solid hex of the given color, with the rest of the enclosing
       rectangle filled with transparent pixels.
     */
   public static Image makeSolidHex(int color, int rowWidth[], int outlineColor){
      int w = rowWidth[rowWidth.length-1];
      int h = 2*rowWidth.length;
      int pix[] = new int[w * h];
      int index = 0;
      int transparentColor= 0;
      for (int row=0; row<rowWidth.length; row++){
         int leftSpace= (w-rowWidth[row])/2;
         int col;
         // fill left and right width space.
         for (col=0; col<leftSpace; col++){
            // top of hex
            pix[row*w+col]= transparentColor;
            pix[row*w+(w-1-col)]= transparentColor;
            // bottom of hex 
            pix[(h-1-row)*w+col]= transparentColor;
            pix[(h-1-row)*w+(w-1-col)]= transparentColor;
         }
         // fill middle with color.
         while (col<leftSpace+rowWidth[row]){
            // top of hex
            pix[row*w+col]= color;
            // bottom of hex
            pix[(h-1-row)*w+col]= color;
            col++;
         }
         // Mark left and right edges with outline color.
/*
         if (outlineColor!=0){
            pix[row*w+leftSpace]= outlineColor;
            pix[(h-1-row)*w+leftSpace]= outlineColor;

            pix[row*w+leftSpace+rowWidth[row]-1]= outlineColor;
            pix[(h-1-row)*w+leftSpace+rowWidth[row]-1]= outlineColor;
         }
*/
      }

      //mark top and bottom with outline color.
      if (outlineColor!=0){
      pix[(h/2)*w+w/2]= outlineColor;
/*
         int leftSpace= (w-rowWidth[0])/2;
         for (int col=leftSpace; col<leftSpace+rowWidth[0]; col++){
            pix[col]= outlineColor;
            pix[(h-1)*w+col]= outlineColor;
         }
*/
      }

      return Toolkit.getDefaultToolkit().createImage(
         new MemoryImageSource(w, h, pix, 0, w)
      );   
   }
}
