//FILE: HexCoordinateConverter.java

/* GEMpire (Generic Empire for the Masses), a java package to support internet oriented 
 * empire-like strategy games and associated utilities.
 * Copyright (C) 1998 Christopher L. Marshall
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * My current email is christopherlmarshall@yahoo.com.  My home address is:
 * 10006 Leamoore Lane,
 * Vienna VA, 22181
 * 
 * It is my hope that this modest effort might assist others in producing Java
 * programs to implement a varied host of empire-like games.
 */

package GEMpire;

/** This class is used to translate mouse click coordinates in a hex map canvas to hex row and column indexes.
   */
public class HexCoordinateConverter{
   static int rowAdjustement[]   ={-1,0,-1,1,1};
   static int columnAdjustement[]={-1,0,0,-1,0};

   int borderMap[][];
   int columnWidth; // width of one hex column 
   int squareHeight; // height of hex's enclosing rectangle

   int convertedRow, convertedColumn;

   public int getRow()       {return convertedRow;}
   public int getColumn() {return convertedColumn;}

   public void convert(int x, int y){
      // get columnWidth divisor and remainder.
      int cwd= x/columnWidth;
      int cwr= x%columnWidth;

      // get squareHeight divisor and remainder
      int shd=y/squareHeight;
      int shr=y%squareHeight;

      convertedRow= 2*shd+rowAdjustement[borderMap[shr][cwr]];
      convertedColumn= cwd+columnAdjustement[borderMap[shr][cwr]];
   }

   public HexCoordinateConverter(int rowWidths[]){
      squareHeight= 2*rowWidths.length;
      columnWidth= rowWidths[0]+rowWidths[rowWidths.length-1];
      borderMap= new int[squareHeight][columnWidth];

      // fill in border map.
      int longRow= rowWidths[rowWidths.length-1];
      int shortRow= rowWidths[0];
      for (int r=0; r<rowWidths.length; r++){
         int leftSpace= (longRow-rowWidths[r])/2;
         int beginRightSpace= leftSpace+rowWidths[r];
         int c;
         // mark left regions
         for (c=0; c<leftSpace; c++){
            borderMap[r][c]= 0;
            borderMap[squareHeight-1-r][c]=3;
         }
         // mark center region
         while (c<beginRightSpace){
            borderMap[r][c]= 1;
            borderMap[squareHeight-1-r][c]=1;
            c++;
         }
         // mark right regions
         for (c=beginRightSpace; c<columnWidth; c++){
            borderMap[r][c]= 2;
            borderMap[squareHeight-1-r][c]= 4;
         }
      }
   }
}
