//FILE: IOUtil.java

/* GEMpire (Generic Empire for the Masses), a java package to support internet oriented  
 * empire-like strategy games and associated utilities. 
 * Copyright (C) 1998 Christopher L. Marshall 
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 *  
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 *  
 * My current email is christopherlmarshall@yahoo.com.  My home address is: 
 * 10006 Leamoore Lane, 
 * Vienna VA, 22181 
 *  
 * It is my hope that this modest effort might assist others in producing Java 
 * programs to implement a varied host of empire-like games. 
 */ 

package GEMpire;

import java.awt.FileDialog;
import java.awt.Frame;
import java.io.*;

public class IOUtil{
   public static boolean skipPastLine(String x, BufferedReader in){
      try {
         String line= in.readLine();
         while (line!=null){
            if (line.trim().equals(x)) return true;
            line= in.readLine();
         }
      }
      catch (IOException e){
      }
      return false;
   }
   public static String getFilename(int fdMode, Frame f){
      FileDialog fd= new FileDialog(f);
      fd.setMode(fdMode);
      fd.setVisible(true);
      String filename= fd.getFile();
      fd.dispose();
      return filename;
   }
   public static PrintWriter openPrintWriterToAppend(String filename) throws IOException{
      RandomAccessFile raf= new RandomAccessFile(new File(filename),"rw");
      raf.seek(raf.length());
      PrintWriter tmp= new PrintWriter(new FileWriter(raf.getFD()));
      return tmp;
   }

   /**  This function takes a line like "1/a;2/b;5/c", and produces an array of String like
     *  {null,"a","b",null,null,null,"c"}.
     *  This is very useful for parsing configuration files.  Returns null if string malformed.
     */
   static String[] parsePropertiesList(
      String line, 
      int    startRange, 
      int    stopRange, 
      char   outerDelimiter, 
      char   innerDelimiter
   ){
      try {
         int numValues= stopRange-startRange+1;
         String sOut[]= new String[numValues];
         for (int i=0; i<numValues; i++) sOut[i]= null;
         DelimitedLineReader outer= new DelimitedLineReader(outerDelimiter);
         DelimitedLineReader inner= new DelimitedLineReader(innerDelimiter);

         outer.setLine(line);
         String valuePair= outer.getNextField();
         if (valuePair!=null)
            if (valuePair.equals("")) return sOut;
         while (valuePair!=null){
            inner.setLine(valuePair);
            String s1= inner.getNextField();
            String s2= inner.getNextField();
            if (s1!=null){
               int i= Integer.parseInt(s1.trim())-startRange;
               if (s2==null) return null;
               sOut[i]= s2;
            }
            valuePair= outer.getNextField();
         }
         return sOut;
      }
      catch (IOException e){}
      catch (NumberFormatException e){}
      return null;
   }
   /** Test of parsePropertiesList()
     */
   public static void main(String args[]) throws Exception{
//      String line="2/a;3/b;7/a";
      String line="";
      String tmp[]= parsePropertiesList(line,0,9,';','/');
      for (int i=0; i<tmp.length; i++){
         System.out.println(i+" "+tmp[i]);
      }
   }
}








