//FILE: ImageArrayPanel.java

/* GEMpire (Generic Empire for the Masses), a java package to support internet oriented 
 * empire-like strategy games and associated utilities.
 * Copyright (C) 1998 Christopher L. Marshall
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * My current email is christopherlmarshall@yahoo.com.  My home address is:
 * 10006 Leamoore Lane,
 * Vienna VA, 22181
 * 
 * It is my hope that this modest effort might assist others in producing Java
 * programs to implement a varied host of empire-like games.
 */

package GEMpire;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

class TestUser implements ImageArrayPanelUser{
   public void rowAndColumnSelected(int row, int column){
      System.out.println("row= "+row+" column= "+column);
   }
}

public class ImageArrayPanel extends Canvas{
   Image im[][];
   int cellHeight, cellWidth;
   int numRows,numCols;   
   ImageArrayPanelUser user;

   ImageArrayPanel(
      Image im[][], 
      int cellWidthSpacing, 
      int cellHeightSpacing,
      ImageArrayPanelUser user
   ){
      this.im= im;
      this.cellWidth= im[0][0].getWidth(this)+cellWidthSpacing;
      this.cellHeight= im[0][0].getHeight(this)+cellHeightSpacing;
      this.user= user;
      numRows= im.length;
      numCols= im[0].length;
      addMouseListener(new MouseAdapter(){
         public void mousePressed(MouseEvent e){
            int row= e.getY()/cellHeight;
            int col= e.getX()/cellWidth;
            if (row>=0&&row<numRows&&col>=0&&col<numCols){
               ImageArrayPanel.this.user.rowAndColumnSelected(row,col);
            }
         }
      });
/*
      addComponentListener(new ComponentAdapter(){
         public void componentShown(ComponentEvent e){
System.err.println("debug: "+e);
         }
         public void componentHidden(ComponentEvent e){
System.err.println("debug: "+e);
         }
         public void componentMoved(ComponentEvent e){
System.err.println("debug: "+e);
         }
         public void componentResized(ComponentEvent e){
System.err.println("debug: "+e);
         }
      });
*/
   }
   public Dimension getPreferredSize(){
      return new Dimension(cellWidth*numCols,cellHeight*numRows);
   }
   public Dimension getMinimumSize(){
      return getPreferredSize();
   }   
   public void paint(Graphics g){
System.err.println("redraw");
      for (int r=0; r<numRows; r++){
         for (int c=0; c<numCols; c++){
            g.drawImage(im[r][c],c*cellWidth,r*cellHeight,this);
         }
      }
   }
   public static void main(String args[]) throws Exception{
      Globals.packageStaticInit(true);
      Frame f= new MainFrame("ImageArrayPanel Test");
      Piece p= new Piece(2,3,0,0);
      Image im= p.getImage();
//      int numRows= Piece.numPossiblePlayers(); 
//      int numCols= Piece.numTypes(); 
      int numRows= 3; 
      int numCols= 3; 
      Image imA[][]= new Image[numRows][numCols];
      for (int r=0; r<numRows; r++){
         for (int c=0; c<numCols; c++){
            imA[r][c]= Piece.getImage(c,r);
         }
      }
      ImageArrayPanel imp= new ImageArrayPanel(imA,2,2,new TestUser()); 
      f.add(imp);
      f.pack();
      f.show();

      BufferedReader in= new BufferedReader(new InputStreamReader(System.in));
      String line= in.readLine();
      ImageArrayPanel imp2= new ImageArrayPanel(imA,2,2,new TestUser()); 
      Frame f2= new MainFrame("ImageArrayPanel Test"); 
      f2.add(imp2); 
      f2.pack(); 
      f2.show(); 

   }
}
