// FILE: MapEditor.java

/* GEMpire (Generic Empire for the Masses), a java package to support internet oriented 
 * empire-like strategy games and associated utilities.
 * Copyright (C) 1998 Christopher L. Marshall
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * My current email is christopherlmarshall@yahoo.com.  My home address is:
 * 10006 Leamoore Lane,
 * Vienna VA, 22181
 * 
 * It is my hope that this modest effort might assist others in producing Java
 * programs to implement a varied host of empire-like games.
 */

package GEMpire;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

public class MapEditor extends MainFrame{
   MenuBar mbar;
   MenuItem newItem,openItem,saveAsItem,saveItem,exitItem;
   MenuItem piecePaletteItem;
   MenuItem resizeMapItem;
   MenuItemListener mlist;

   int curPieceType=0; 
   int curTerrainType=0; 
   int playerNumber=0;
   String brushType="terrain";
   String filename;
   PiecePalette pp;

   Map m;
   int numRows= 20;
   int numColumns= 10;

   MapEditor(){
      super("Map Editor");

      mlist= new MenuItemListener();
      mbar= new MenuBar();
      Menu fileMenu= new Menu("File",true);
      fileMenu.add(newItem=    new MenuItem("New"));
      fileMenu.add(openItem=   new MenuItem("Open"));
      fileMenu.add(saveAsItem= new MenuItem("Save As"));
      fileMenu.add(saveItem=   new MenuItem("Save"));
      fileMenu.addSeparator();
      fileMenu.add(exitItem=   new MenuItem("Exit"));
      newItem.addActionListener(mlist);
      openItem.addActionListener(mlist);
      saveAsItem.addActionListener(mlist);
      saveItem.addActionListener(mlist);
      exitItem.addActionListener(mlist);

      Menu brushMenu= new Menu("Brush",true);
      brushMenu.add(piecePaletteItem= new MenuItem("Piece Palette"));
      piecePaletteItem.addActionListener(mlist);

      Menu commandMenu= new Menu("Command",true);
      commandMenu.add(resizeMapItem= new MenuItem("Resize Map"));
      resizeMapItem.addActionListener(mlist);

      mbar.add(fileMenu);
      mbar.add(brushMenu);
      mbar.add(commandMenu);


      ImageArrayPanelUser user1= new ImageArrayPanelUser(){
         public void rowAndColumnSelected(int row, int column){
            brushType="terrain";
            curTerrainType=column;
         }
      };
      ImageArrayPanelUser user2= new ImageArrayPanelUser(){
         public void rowAndColumnSelected(int row, int column){
            playerNumber= row;
            brushType= "piece";
            curPieceType= column;
         }
      };
      pp= new PiecePalette(user1,user2);

      setMenuBar(mbar);
      setSize(200,200);
      setBackground(Color.white);
   }
   class MenuItemListener implements ActionListener{
      public void actionPerformed(ActionEvent event){
         MenuItem item= (MenuItem)event.getSource();
         if (item==exitItem){
            dispose();
            Globals.exit("");
         }
         else if (item==newItem){
               try {
                  if (m!=null) remove(m);
                  m=new Map((MapEditor.this),numRows,numColumns,8,4);
                  initMap();
               } catch (BadRowAndColumnValuesException e) {Globals.exit(e.toString());}
         }
         else if (item==saveItem){
            if (filename==null){
               filename= IOUtil.getFilename(FileDialog.SAVE,MapEditor.this);
            }
            if (filename!=null&&m!=null){
               try {
                  Writer out= new FileWriter(filename);
                  m.write(out);
                  out.close();
               }
               catch (IOException e) {Globals.exit(e.toString());}
            }
         }
         else if (item==saveAsItem){
            if (m!=null){
               filename= IOUtil.getFilename(FileDialog.SAVE,MapEditor.this);
               if (filename!=null){
                  try {
                     Writer out= new FileWriter(filename);
                     m.write(out);
                     out.close();
                  }
                  catch (IOException e) {Globals.exit(e.toString());}
               }
            }
         }
         else if (item==openItem){
            filename= IOUtil.getFilename(FileDialog.LOAD,MapEditor.this);
            if (filename!=null){
               try {
                  if (m!=null){
                     removeAll();
                     m.removeFromFrame();
                  }
                  Reader in= new FileReader(filename);
                  m= new Map(MapEditor.this,in);
                  in.close();
                  initMap();
                  System.out.println("memory left= "+SystemUtil.fullGC());
               }
               catch (FileNotFoundException e) {Globals.exit(e.toString());}
               catch (BadRowAndColumnValuesException e) {Globals.exit(e.toString());}
               catch (IOException e) {Globals.exit(e.toString());}
            }
         }
         else if (item==piecePaletteItem){
            pp.setLocation(MapEditor.this.getLocation());
            pp.setVisible(true);
         }
         else if (item==resizeMapItem){
            int sizes[]={3,3};
            String fieldNames[]={"number of rows","number of columns"};
            String initialValues[]={""+numRows,""+numColumns};
            StringArrayDialog sad= new StringArrayDialog(
               MapEditor.this,
               "Map Size Dialog",
               fieldNames,
               initialValues,
               sizes
            );
            sad.setLocation(MapEditor.this.getLocation());
            sad.setVisible(true);
            String values[]= sad.getStrings();
            int newRows, newCols;
            while (true){
               try {
                  newRows= Integer.parseInt(values[0].trim());
                  newCols= Integer.parseInt(values[1].trim());
                  if (newRows%2==1) newRows++;
                  if (newCols%2==1) newCols++;
                  m.resizeGrid(newRows,newCols);
                  numRows= newRows; numColumns= newCols;
                  initMap();
                  return;
               }
               catch (NumberFormatException e){
                  System.out.println("badly formed");
               }
            }
         }
      }
   }

   void initMap(){
      m.setDisplaySize(8,4,this);
      m.setRowAndColumnSelectedListener(new RowAndColumnSelectedListener(){
         public void selected(int row, int column){
            if (brushType.equals("piece")){
               Piece p= m.getPieceAt(row,column);
               Piece tmp= new Piece(curPieceType,playerNumber,row,column);
               if (p==null)
                  m.attachPiece(new Piece(curPieceType,playerNumber,row,column));
               else {
                 if (p.canCarry(tmp))
                    p.attachPiece(tmp);
                 else 
                    m.detachPiece(p);
               }
            }
            // Permute terrain type, skipping UNEXPLORED type.
            else if (brushType.equals("terrain")){
   
            if (m.getPieceAt(row,column)!=null) return;
/*
               int tmp= m.getTerrain(row,column);
               do {
                  tmp= (tmp+1)%TerrainTypes.numTypes;
               } while (tmp==TerrainTypes.UNEXPLORED);
*/
               m.setTerrain(row,column,curTerrainType);
            }
            else {
               Globals.exit("MapEditor.java: invalid brush type, this shouldn't happen");
            }
            m.drawCell(row,column);
         }
      });
      add(m,"Center");
      pack();
   }

   public static void main(String args[]){
      Globals.packageStaticInit(true);
      MapEditor me= new MapEditor();
      me.setVisible(true);
   }
}





