//FILE: MovementReport.java 
 
/* GEMpire (Generic Empire for the Masses), a java package to support internet oriented  
 * empire-like strategy games and associated utilities. 
 * Copyright (C) 1998 Christopher L. Marshall 
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 *  
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 *  
 * My current email is christopherlmarshall@yahoo.com.  My home address is: 
 * 10006 Leamoore Lane, 
 * Vienna VA, 22181 
 *  
 * It is my hope that this modest effort might assist others in producing Java 
 * programs to implement a varied host of empire-like games. 
 */ 
package GEMpire;

import java.io.PrintStream;

public class MovementReport{
   Piece p;
   Piece p2;
   int   defender;
   Piece reportP1;
   Piece reportP2;

   int[]     from_rc; // source cell row and column
   int[]     to_rc;   // destination cell row and column
   boolean[] sourceWasVisibleTo;
   boolean[] destinationWasVisibleTo;
   boolean[] sourceIsVisibleTo;
   boolean[] destinationIsVisibleTo;

   boolean moveOccurred   = false;
   int     assumeVisibleTo= -1;
   boolean combatOccurred = false;

   MovementReport(Map m, Piece p, int row, int col, boolean players[]){
      this.p                 = p;
      p2                     = m.getPieceAt(row,col);
      from_rc                = p.getRowAndColumn();
      to_rc                  = new int[2];
      to_rc[0]               = row;
      to_rc[1]               = col;

      sourceWasVisibleTo          = m.getVisibleStatus(from_rc[0],from_rc[1],players);
      destinationWasVisibleTo     = m.getVisibleStatus(to_rc[0],to_rc[1],players);
      reportP1                    = (Piece)(p.clone());
      reportP2                    = m.getPieceAt(row,col);
      if (reportP2!=null) reportP2= (Piece)(reportP2.clone());

      // set defender in case combat occurs.
      defender= -1;
      if (reportP2!=null){
         if (!reportP2.isNeutral()){
            defender= reportP2.getOwner();
         }
      }
   }

   // Methods Called by PieceMover to store results.
      public void storeResults(Map m, boolean players[]){
         sourceIsVisibleTo     = m.getVisibleStatus(from_rc[0],from_rc[1],players);      
         destinationIsVisibleTo= m.getVisibleStatus(to_rc[0],to_rc[1],players);     
      }
      public void setMoveOccurred(boolean b){
         moveOccurred= b;
      }
      public void setCombatOccurred(){
         combatOccurred= true;
      }

   // Methods called by EmpireServer to retrieve results.
      public boolean moveOccurred(){
         return moveOccurred;
      }
      public boolean combatOccurred(){
         return combatOccurred;
      }
      public boolean[] getSourceVisibilityBeforeMove(){
         return sourceWasVisibleTo;
      }
      public boolean[] getDestinationVisibilityBeforeMove(){
         return destinationWasVisibleTo;
      }
      public boolean[] getSourceVisibilityAfterMove(){
         return sourceIsVisibleTo;
      }
      public boolean[] getDestinationVisibilityAfterMove(){
         return destinationIsVisibleTo;
      }
      /** If the player moving the piece has his piece die, he still
        * deserves to see what happened in the from-cell and to-cell of
        * the move.  Ditto for the defeated prior occupant of the to-cell.
        * This method returns the player id of the player in this situation,
        * if any, or -1 otherwise.  In any given scenario, there is at
        * most one such player.
        */
      public int shouldAssumeVisibleTo(){
         if (!p.isAlive()){
            return p.getOwner();
         }
         if (p2!=null) 
            if (!p2.isAlive()){
               return p2.getOwner();
            }
         return -1;
      }
      public Piece getReportP1(){return reportP1;}
      public Piece getReportP2(){return reportP2;}
      public int getDefender(){return defender;} // a -1 indicates a neutral defender
}
