//FILE: PieceDatabase.java

/* GEMpire (Generic Empire for the Masses), a java package to support internet oriented 
 * empire-like strategy games and associated utilities.
 * Copyright (C) 1998 Christopher L. Marshall
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * My current email is christopherlmarshall@yahoo.com.  My home address is:
 * 10006 Leamoore Lane,
 * Vienna VA, 22181
 * 
 * It is my hope that this modest effort might assist others in producing Java
 * programs to implement a varied host of empire-like games.
 */

package GEMpire;

import java.awt.*;
import java.io.*;

public class PieceDatabase{
   static int defaultNumPossiblePlayers= 12;
   static int defaultNumPieceTypes= 12;
   static String pieceInfoFile="pieceinfo.txt";

   int   numPieceTypes;
   int   numPossiblePlayers;

   PieceInfo pi[];

   Image pieceImagesByType[];
   Image pieceImages[][];

   Color   playerColors[];

/*
   public String[] getPieceNames(){
      String tmp[]= new String[pi.length];
   }
*/

   public String name(int type){return pi[type].name;}
   public int numPieceTypes(){return numPieceTypes;}
   public int numPossiblePlayers(){return numPossiblePlayers;}

   public boolean isNeutral(int owner){return (owner==0);}
   public int     movesPerTurn(int type){return pi[type].movesPerTurn;}
   public int     hitPoints(int type){return pi[type].damage;}
   public int     shotDamage(int type){return pi[type].shotDamage;}
   public int     range(int type){return pi[type].range;}
   public int     productionTime(int typea, int typeb){return pi[typea].productionTime[typeb];}
   public boolean canProduce(int typea, int typeb){return (pi[typea].productionTime[typeb]!=-1);}
   public int     buildTime(int typea, int typeb){return pi[typea].buildTime[typeb];}
   public boolean canBuild(int typea, int typeb){return pi[typea].buildTime[typeb]!=-1;}
   public Color   playerColor(int playerNumber){return playerColors[playerNumber];}
   /** Can this piece occupy this type of terrain? */
   public boolean canMoveOver(int type, int terrain){return (pi[type].movementCost[terrain]!=-1);}
   public int     movementCost(int type, int terrain){return pi[type].movementCost[terrain];}
   public boolean canCarry(int typea, int typeb){return pi[typea].carryingCost[typeb]!=-1;}
   public int     carryingCost(int typea, int typeb){return pi[typea].carryingCost[typeb];}
   public int     carryingCapacity(int type){return pi[type].carryingCapacity;}
   public boolean canSee(int typea, int typeb){return pi[typea].canSee[typeb];}
   /** As of 7/1998, Cities are the only fixed pieces.  The concept of a fixed piece is that the
     * piece represents a special cell and is more of a terrain feature than a piece.  
     * The distinction is important because of the selective visibility mechanism.  Once a cell
     * has been explored, the terrain features should always be 'visible' to the player who
     * explored.  This is not true for mobile enemy pieces, where a player can not see them in 
     * any sense unless one of her pieces is adjacent to it and can see pieces of that type.
     */
   public boolean isFixed(int type){return pi[type].isFixed;}
   public int attackType(int typea, int typeb){
      if (((typea==3)||(typea==4))&&(typeb==0)) return Piece.BOMBING_ATTACK;
      return Piece.NORMAL_ATTACK;
   }
   public double bombingSuccessRate(int typea, int typeb){
      if ((typea==3)&&(typeb==0)) return (1.0/3.0);
      else if (typea==4&&typeb==0) return 0.5;
      Globals.exit("PieceDatabase.bombingSuccessRate(): invalid type combo ("+typea+","+typeb+")");
      return 0.0;
   }
   public boolean canAttack(int pieceA, int terrainA, int pieceB, int terrainB){
       return pi[pieceA].canAttack(terrainA, pieceB, terrainB);
   }
   public double  defenseFactor(int pieceA, int terrainA, int pieceB, int terrainB){
      return pi[pieceA].defenseFactor(terrainA,pieceB,terrainB);
   }
   public double  attackFactor(int pieceA, int terrainA, int pieceB, int terrainB){
      return pi[pieceA].attackFactor(terrainA,pieceB,terrainB);
   }
   
   final public Image getImage(int type, int player){
      return pieceImages[type][player];
   }

   public PieceDatabase(boolean graphicsEnabled){
      numPieceTypes= defaultNumPieceTypes;
      numPossiblePlayers= defaultNumPossiblePlayers;

      readPieceInfoFile();

      playerColors= new Color[numPossiblePlayers];
      playerColors[0]= Color.gray;
      playerColors[1]= Color.white;
      playerColors[2]= Color.red;
      playerColors[3]= Color.black;
      playerColors[4]= Color.magenta;
      playerColors[5]= Color.magenta.darker();
      playerColors[6]= Color.yellow.darker();
      playerColors[7]= Color.pink;
      playerColors[8]= Color.cyan;
      playerColors[9]= Color.cyan.darker();
      playerColors[10]= Color.orange;
      playerColors[11]= Color.orange.darker();

      if (graphicsEnabled){
         pieceImagesByType= new Image[numPieceTypes];
         pieceImages= new Image[numPieceTypes][numPossiblePlayers];
         for (int i=0; i<numPieceTypes; i++)
            pieceImages[i]= new Image[numPossiblePlayers];

         try {loadImages();} 
         catch (InterruptedException e){
             Globals.exit("PieceDatabase(): interrupted while trying to load images");
         }
      }
      else {
         pieceImagesByType= null;
         pieceImages= null;
      }

   }

   void loadImages() throws InterruptedException{
      Frame f= new Frame();
      MediaTracker m= new MediaTracker(f);
      int black      = Color.black.getRGB();

      // Get black prototype piece images.
      for (int i=0; i<numPieceTypes; i++){
         Image im= GraphicsUtil.getImage("images/"+pi[i].name+".gif");
         if (im==null) 
            Globals.exit("PieceDatabase.loadImages(): could not load image for "+pi[i].name);
         m.addImage(im,0);
         m.waitForID(0);
         pieceImagesByType[i]= im;

         // Recolor for every player.
         for (int j=0; j<numPossiblePlayers; j++){
            // Make white transparent.
            Image tmp= GraphicsUtil.changeColor(im,Color.white.getRGB(),0,f);

            // replace black with player color.
            tmp= GraphicsUtil.changeColor(
               tmp,
               Color.black.getRGB(),
               playerColors[j].getRGB(),
               f
             );

            pieceImages[i][j]= tmp;
         }
      }
      f.dispose();
   }
   
   void readPieceInfoFile(){
      // open piece info file
      BufferedReader in=null;
      try {in= new BufferedReader(new FileReader(pieceInfoFile));}
      catch (FileNotFoundException e){
         Globals.exit(
            "PieceDatabase.readPieceInfoTable(): could not open file "+
            pieceInfoFile
         );
      }

      pi= new PieceInfo[numPieceTypes];

      try {
         String line= in.readLine();
         int state= 0;
         int i=0;
         while (line!=null&&state!=3){
            line= line.trim();
            if (!line.equals("")){
               if (state==0){
                  if (line.equals("PieceTableBegin")){
                     state= 1;
                  }
               }
               else if (state==1){
                  if (line.indexOf("Type")!=-1) state= 2;
               }
               else if (state==2){
                  if (line.equals("PieceTableEnd")){
                     if (i<numPieceTypes)
                        Globals.exit(
                           "PieceDatabase.readPieceInfoFile(): expected to read "+numPieceTypes+
                           " piece descriptions in "+pieceInfoFile+
                           " but only found "+i
                        );
                     state= 3;
                  }
                  else {
                     if (i>=numPieceTypes)
                        Globals.exit(
                           "PieceDatabase.readPieceInfoFile(): too many piece descriptions"+
                           "found in"+pieceInfoFile+
                            ".  Only expected to find "+numPieceTypes
                        );

                     // Read piece description.
                     try {
                        if (PieceInfo.isPieceDescription(line)){
                           pi[i++]= new PieceInfo(line,numPieceTypes);
                        }
                     } 
                     catch (IOException e){
                        Globals.exit(
                           "PieceDatabase.readPieceInfoFile(): could not parse piece "+
                           "description in "+pieceInfoFile+
                           " : "+line+"\n\r"+e
                        );
                     }
                  }
               }
               else if (state==3){
               }
            }
            line= in.readLine();
         }
         in.close();
      }
      catch (IOException e){
         Globals.exit("PieceDatabase.readPieceInfoFile(): "+e);
      }
   }

   public static void main(String args[]){
      PieceDatabase pdb= new PieceDatabase(true);
      PrintWriter out= new PrintWriter(new OutputStreamWriter(System.out));
      for (int i=0; i<pdb.numPieceTypes; i++){
         pdb.pi[i].reportOnCombatProperties(out, pdb.pi);
         out.flush();
      }
      System.exit(0);
   }
}


