//FILE: RulerBar.java

/* GEMpire (Generic Empire for the Masses), a java package to support internet oriented 
 * empire-like strategy games and associated utilities.
 * Copyright (C) 1998 Christopher L. Marshall
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * My current email is christopherlmarshall@yahoo.com.  My home address is:
 * 10006 Leamoore Lane,
 * Vienna VA, 22181
 * 
 * It is my hope that this modest effort might assist others in producing Java
 * programs to implement a varied host of empire-like games.
 */

package GEMpire;

import java.awt.*;
import java.awt.event.*;

public class RulerBar extends Canvas{
   static int widthPadding= 2;
   Font f;
   FontMetrics fm;
   int offset;
   int spacing;
   int first;
   int last;
   int max;
   int numSlots;
   Dimension d= new Dimension(10,20);
   boolean horizontal;

   public RulerBar(Font f, int offset, int spacing, int first, int last, int max, boolean horizontal)
      throws RulerBarSpacingTooSmallException
   {
      super();
      if (f==null) f= new Font("Dialog",Font.PLAIN,9);
      this.f= f;
      setBackground(Color.green.darker());

      this.offset= offset;
      this.spacing= spacing;
      this.first= first;
      this.last= last;
      numSlots= last-first+1;
      this.max= max;
      this.horizontal= horizontal;

      fm= Toolkit.getDefaultToolkit().getFontMetrics(f);
      setSpacing(fm);
   }

   public void setSpacing(FontMetrics fm)
      throws RulerBarSpacingTooSmallException
   {
      int fontHeight= fm.getAscent();//+fm.getDescent();
      int maxDigits= (""+max).length();
      if (horizontal){
         int requiredSpacing= maxDigits*fm.charWidth('A');
         if (spacing<requiredSpacing)
            throw new RulerBarSpacingTooSmallException(
               "spacing= "+spacing+" and required spacing= "+requiredSpacing
            );
         d= new Dimension(spacing*(last-first+1),fontHeight);
      }
      else {
         if (spacing<fontHeight) 
            throw new RulerBarSpacingTooSmallException(
               "spacing= "+spacing+" and requiredSpacing= "+fontHeight
            );
         d= new Dimension(maxDigits*fm.charWidth('A'),spacing*(last-first+1));
      }
   }

   public void setRange(int first, int numSlots){
      this.first= first;
      this.numSlots= numSlots;
      last= first+numSlots-1;
      repaint();
   }

   public Dimension getPreferredSize(){return d;}
   public Dimension getMinimumSize(){return d;}

   public void paint(Graphics g){
      FontMetrics fm1= g.getFontMetrics();
      if (fm1!=fm){
         try {setSpacing(fm);}
         catch (RulerBarSpacingTooSmallException e){
            Globals.exit(e.toString());
         }
      }
      
      int y= fm.getAscent();
      if (horizontal){
         for (int i=0; i<numSlots; i++){
            g.drawString(""+(i+first),i*spacing+offset,y);
         }
      }
      else {
         for (int i=0; i<numSlots; i++){
            g.drawString(""+(i+first),0,i*spacing+y+offset);
         }
      }
   }

   public static void main(String args[])
      throws RulerBarSpacingTooSmallException
   {
      Frame f= new MainFrame("FontTest");
      Canvas c= new RulerBar(null,0,30,0,15,100,true);
      System.out.println(c.getPreferredSize());
      f.add(c,"West");
      f.pack();
      f.show();
   }
}
