//FILE: SystemUtil.java 
 
/* GEMpire (Generic Empire for the Masses), a java package to support internet oriented  
 * empire-like strategy games and associated utilities. 
 * Copyright (C) 1998 Christopher L. Marshall 
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 *  
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 *  
 * My current email is christopherlmarshall@yahoo.com.  My home address is: 
 * 10006 Leamoore Lane, 
 * Vienna VA, 22181 
 *  
 * It is my hope that this modest effort might assist others in producing Java 
 * programs to implement a varied host of empire-like games. 
 */ 
 
package GEMpire;

import java.awt.*;
import java.io.*;
class APanel extends Panel{
   int x[];
   int y[]= {0};
   APanel(){
      super();
      x= new int[10000];
   }
}

public class SystemUtil{
   final public static int rint(int N){return ((int)(Math.random()*N))%N;}
   static int []NChooseK(int N, int K){
      if (N<K){
         System.out.println("NChooseK("+N+","+K+"): N shouldn't be less than K");
         System.exit(1);
      }

      int x[]= new int[N];
      for (int i=1; i<N; i++) x[i]= i;

      // preform n random swaps
      for (int i=0; i<N; i++){
         // choose 2 elements at random to swap.
         int j=rint(N);
         int k=rint(N);

         // swap j and k in x[]
         int tmp= x[j];
         x[j]= x[k];
         x[k]= tmp;
      }

      // copy first k elements to return to caller
      int y[]= new int[K];
      for (int i=0; i<K; i++){
         y[i]= x[i];
      }

      return y;
   }
   public static String userLine() throws IOException{ 
      BufferedReader in= new BufferedReader(new InputStreamReader(System.in)); 
      return in.readLine(); 
   } 
   public static void userPause(){ 
      try {userLine();} catch (IOException e){} 
   } 
   public static long fullGC(){
      Runtime rt= Runtime.getRuntime();
      long isfree= rt.freeMemory();
      long wasfree;
      do {
         wasfree= isfree;
         rt.gc();
         isfree= rt.freeMemory();
      } while (isfree>wasfree);
      rt.runFinalization();
      return isfree;
   }
   public static long getFree(){
      return Runtime.getRuntime().freeMemory();
   }

   /** The following test routines are used to detect memory leaks in the Map class.
     */

   public static void test1() throws Exception{
      System.out.println("memory left= "+SystemUtil.fullGC());
      Frame f= new Frame(); f.setLayout(new BorderLayout());

      Map m1= new Map(null,100,100,10,10);
      f.add(m1,"Center");
      System.out.println("memory left= "+SystemUtil.fullGC());
      m1.removeFromFrame();
      f.remove(m1);
      m1= null;

      f.dispose();

      System.out.println("memory left= "+SystemUtil.fullGC());
      System.exit(0);
   }
   public static void test2() throws Exception{
      int N= 10;

      Frame f[]= new Frame[N];
      APanel p[]=  new APanel[N];
      Object x[]= new Object[N];

      System.out.println("memory left= "+SystemUtil.fullGC());
      for (int i=0; i<N; i++){
         f[i]= new Frame();
         p[i]= new APanel();
         x[i]= p[i].y;
         f[i].setLayout(new BorderLayout());
         f[i].add(p[i],"Center");
         System.out.println("memory left after add= "+SystemUtil.fullGC());
      }

      for (int i=0; i<N; i++){
         f[i].removeAll();
         p[i]= null;
         f[i].dispose();
         f[i]= null;
         System.out.println("memory left after remove= "+SystemUtil.fullGC());
      }
      System.exit(0);
   }
   public static void test3() throws Exception{
      int N= 15;

      Frame f= new Frame();
      Map p[]=  new Map[N];

      System.out.println("memory left= "+SystemUtil.fullGC());
      for (int i=0; i<N; i++){
         p[i]= new Map(f,30,30,10,10);
         //p[i].setDisplaySize(10,10,f);
         f.add(p[i]);
         System.out.println("memory left after add= "+SystemUtil.fullGC());
      }

      for (int i=0; i<N; i++){
         f.removeAll();
         p[i].removeFromFrame();
         p[i]= null;
         System.out.println("memory left after remove= "+SystemUtil.fullGC());
      }

      f.dispose();
      f= null;
      System.out.println("memory left after final dispose= "+SystemUtil.fullGC());
      System.exit(0);
   }
   public static void test4() throws Exception{
      int N= 15;

      Frame f= new Frame();
      Map p[]=  new Map[N];

      System.out.println("memory left= "+SystemUtil.fullGC());
      for (int j=1; j<N; j++){
         for (int i=0; i<j; i++){
            p[i]= new Map(f,30,30,10,10);
            f.setLayout(new FlowLayout());
            f.add(p[i]);
         }

         for (int i=0; i<j; i++){
            f.removeAll();
            p[i].removeFromFrame();
            p[i]= null;
         }

         System.out.println("memory left after "+j+" iterations= "+SystemUtil.fullGC());
      }

      for (int j=1; j<N; j++){
         for (int i=0; i<j; i++){
            p[i]= new Map(f,30,30,10,10);
            f.setLayout(new FlowLayout());
            f.add(p[i]);
         }

         for (int i=0; i<j; i++){
            f.removeAll();
            p[i].removeFromFrame();
            p[i]= null;
         }

         System.out.println("memory left after "+j+" iterations= "+SystemUtil.fullGC());
      }

      f.dispose();
      f= null;
      System.out.println("memory left after final dispose= "+SystemUtil.fullGC());
      System.exit(0);
   }
   public static void test5() throws Exception{
      int N= 50;

      Frame f= new Frame();
      Map m;

      System.out.println("memory left= "+SystemUtil.fullGC());
      for (int i=0; i<N; i++){
         m= new Map(f,new FileReader("test2.map"));
         m.setDisplaySize(8,4,f);
         m.setRowAndColumnSelectedListener(new RowAndColumnSelectedListener(){
            public void selected(int row, int column){
               System.out.println("row= "+row+" column= "+column);
            }
         });
         f.add(m);
         f.pack();
         f.setVisible(true);
         userPause();
         f.removeAll();
         m.removeFromFrame();
         m= null;
         System.out.println("memory left after add/remove= "+SystemUtil.fullGC());
      }
      System.exit(0);
   }
   public static void test6() throws Exception{
      int N= 50;

      Frame f= new Frame();
      f.setVisible(true);

      System.out.println("memory left= "+SystemUtil.fullGC());
      for (int i=0; i<N; i++){
         IOUtil.getFilename(FileDialog.LOAD,f);
         userPause();
         System.out.println("memory left after add/remove= "+SystemUtil.fullGC());
      }
      System.exit(0);
   }
   public static void test7(){
      for (int i=0; i<5; i++){
         int y[]= NChooseK(10,3);
         for (int j=0; j<y.length; j++)
            System.out.print(y[j]+" ");
         System.out.println();
      }
   }
   public static void main(String args[]) throws Exception{
      test7();
   }
}
